using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Management;
using System.Threading;
using System.IO;

namespace CachingThreadWaitTests
{
    /// <summary>
    /// Summary description for CachingInstrumentationThreadTestFixture
    /// </summary>
    [TestClass]
    public class CachingInstrumentationThreadTestFixture
    {
        private CacheManager itemCache;
        String wmiText = "";
        bool wmiLogged = false;
        public static bool removed = false;
        
        public CachingInstrumentationThreadTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize() {
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            if (itemCache != null)
                itemCache.Flush();
        }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        
        [TestMethod]
        public void ScavengedItemsCounterTest()
        {
            string counterName = "Cache Scavenged Items/sec";
            int loggedCount = 0;

            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            int initialCount = GetCounterValue(counterName);

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            Thread.Sleep(1000);
            

            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 3, initialCount);
        }

        [TestMethod]
        public void ScavengedItemsWMITest()
        {
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(2, "Toy2", 22);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(3, "Toy3", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(4, "Toy4", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(5, "Toy5", 13);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(6, "Toy6", 16);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.NotRemovable, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(7, "Toy7", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Low, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(8, "Toy8", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.None, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(9, "Toy9", 11);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            itemDetails = new ItemDetails(10, "Toy10", 21);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));

            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM CacheScavengedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();

            itemDetails = new ItemDetails(11, "Toy11", 23);
            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.Normal, null, new AbsoluteTime(DateTime.Now.AddMinutes(10)));
            Thread.Sleep(1000);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("CacheScavengedEvent") == -1)
            {
                Assert.Fail();
            }

            
        }

        [TestMethod]
        public void CallbackFailedWMITest()
        {
            removed = false;
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);
            
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM CacheCallbackFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), new FileDependency("FileForCaching.txt"));
            Assert.AreEqual(false, removed);
            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            Thread.Sleep(1000);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            Assert.IsFalse(removed);
            if (wmiText.IndexOf("CacheCallbackFailureEvent") == -1)
            {
                Assert.Fail();
            }


        }

        [TestMethod]
        public void CallbackFailedEventLogTest()
        {
            removed = false;
            itemCache = CacheFactory.GetCacheManager("DefaultCacheManager");
            ItemDetails itemDetails = new ItemDetails(1, "Toy1", 25);

            itemCache.Add(itemDetails.ItemId.ToString(), itemDetails, CacheItemPriority.High, new ItemRefreshAction(), new FileDependency("FileForCaching.txt"));
            Assert.AreEqual(false, removed);
            File.SetLastWriteTime("FileForCaching.txt", DateTime.Now);
            Thread.Sleep(1000);

            Assert.IsFalse(removed);

            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("DefaultCacheManager")))
                    Assert.Fail("Event Not logged");
            }
        }

        private int GetCounterValue(string counterName)
        {
            string categoryName = "Enterprise Library Caching Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("DefaultCacheManager");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }

        [Serializable]
        public class ItemRefreshAction : ICacheItemRefreshAction
        {
            public void Refresh(string removedKey, object expiredValue, CacheItemRemovedReason removalReason)
            {
                throw new ApplicationException("Callback Event Exception");

            }
        }
    }


}
