using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Caching.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;

namespace CachingThreadWaitTests
{
    public class CachingDictionarySource
    {
        public DictionaryConfigurationSource BuildDictionarySourceSection()
        {
            DictionaryConfigurationSource sections = new DictionaryConfigurationSource();
            CacheManagerSettings settings = new CacheManagerSettings();

            settings.DefaultCacheManager = "InMemoryCacheManager";

            #region "Backing Stores"
            CacheStorageData csd = new CacheStorageData();
            csd.Name = "inMemoryDic";
            csd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations.NullBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching");
            settings.BackingStores.Add(csd);

            csd = new CacheStorageData(); 
            csd.Name = "inMemoryThreadDic";
            csd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations.NullBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching");
            settings.BackingStores.Add(csd);

            IsolatedStorageCacheStorageData iscsd = new IsolatedStorageCacheStorageData();
            iscsd.Name = "IsolatedStorageDic";
            iscsd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Caching.BackingStoreImplementations.IsolatedStorageBackingStore, Microsoft.Practices.EnterpriseLibrary.Caching");
            iscsd.PartitionName = "EntLib1";
            settings.BackingStores.Add(iscsd);

            #endregion

            #region "Cache Managers"
            CacheManagerData cmd = new CacheManagerData();
            cmd.Name = "InMemoryCacheManager";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "inMemoryDic";
            settings.CacheManagers.Add(cmd);

            cmd = new CacheManagerData();
            cmd.Name = "InMemoryThreadCacheManager";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10000;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "inMemoryThreadDic";

            settings.CacheManagers.Add(cmd);

            cmd = new CacheManagerData();
            cmd.Name = "IsolatedCacheManagerDic";
            cmd.ExpirationPollFrequencyInSeconds = 1;
            cmd.MaximumElementsInCacheBeforeScavenging = 10;
            cmd.NumberToRemoveWhenScavenging = 3;
            cmd.CacheStorage = "IsolatedStorageDic";

            settings.CacheManagers.Add(cmd);

            #endregion 

            sections.Add("cachingConfiguration", settings);

            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            sections.Add("instrumentationConfiguration", ics);
            return sections;
        }
    }
}
