using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common ;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.ObjectBuilder;       

namespace ConfigurationCoreTests.Configuration
{
    [Assembler(typeof(ItemAssembler))]
    public class ItemData : NameTypeConfigurationElement
    {
        private const string priceProperty = "price";
        private const string quantityProperty = "quantity";
        
        /// <summary>
        /// Initialize a new instance of the <see cref="CacheManagerData"/> class.
        /// </summary>
        public ItemData()
        {
        }


        public ItemData(string name, Type type, decimal price, int quantity)
            : base(name, type)
        {
            this.Price = price;
            this.Quantity = quantity;
        }

        /// <summary>
        /// Price of item
        /// </summary>
        [ConfigurationProperty(priceProperty, IsRequired = true)]
        public decimal Price
        {
            get { return (decimal)base[priceProperty]; }
            set { base[priceProperty] = value; }
        }

        [ConfigurationProperty(quantityProperty, IsRequired  = true)]
        public int Quantity
        {
            get { return (int)base[quantityProperty]; }
            set { base[quantityProperty] = value; }
        }
                
    }

    class ItemAssembler : IAssembler<IItem, ItemData>
    {
        public IItem Assemble(IBuilderContext context, ItemData  objectConfiguration, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            IItem createdObject = new CommonItem(objectConfiguration.Price, objectConfiguration.Quantity);      
            return createdObject;
        }

    }
}
