using System;
using System.Collections.Generic;
using System.Text;
using ManagementInstrumentation = System.Management.Instrumentation.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
 
namespace ConfigurationCoreTests
{
    [PerformanceCountersDefinition("Test Item Counters", "ItemsHelpResource")]
    [EventLogDefinition("Application", "Enterprise Library Test")]
    [HasInstallableResources] 
    public class ItemInstrumentationListener : InstrumentationListener
    {
        static EnterpriseLibraryPerformanceCounterFactory factory = new EnterpriseLibraryPerformanceCounterFactory();

        [PerformanceCounter("Item Categories Calculated/sec", "ItemsCalculated", PerformanceCounterType.RateOfCountsPerSecond32)]
        private EnterpriseLibraryPerformanceCounter itemsRaised;

        public ItemInstrumentationListener(bool performanceCountersEnabled,
                                              bool eventLoggingEnabled,
                                              bool wmiEnabled)
            : base(performanceCountersEnabled, eventLoggingEnabled, wmiEnabled, new AppDomainNameFormatter())
        {
		}

        public ItemInstrumentationListener(string instanceName,
                                              bool performanceCountersEnabled,
                                              bool eventLoggingEnabled,
                                              bool wmiEnabled)
            : base(instanceName, performanceCountersEnabled, eventLoggingEnabled, wmiEnabled, new AppDomainNameFormatter())
        {
        }

        [InstrumentationConsumer("ItemCalculated")]
        public void ItemCalculated(object sender,EventArgs e)
        {
            if (PerformanceCountersEnabled)
            {
                if (itemsRaised != null) itemsRaised.Increment();  
            }
        }

        [InstrumentationConsumer("CalculationError")]
        public void CalculationError(object sender, ErrorEventArgs e)
        {
            if (WmiEnabled)
            {
               ManagementInstrumentation.Fire(new ItemCalculatedEvent(e.ErrorMessage,e.ExceptionMessage));      
            }
            if (EventLoggingEnabled)
            {
                EventLog.WriteEntry(GetEventSourceName(), e.ErrorMessage,EventLogEntryType.Error );   
            }
        }

        protected override void CreatePerformanceCounters(string[] instanceNames)
        {
            itemsRaised = factory.CreateCounter("Test Item Counters", "Item Categories Calculated/sec", instanceNames);
        }

    }
}
