using System;
using System.Text;
using System.Collections.Generic;
using System.Configuration;
using System.Xml;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using ConfigurationCoreTests.Configuration;       
namespace ConfigurationCoreTests
{
    /// <summary>
    /// Summary description for SingletonObjectTestFixture
    /// </summary>
    [TestClass]
    public class SingletonObjectTestFixture
    {
        private bool carConfigNotificationFlag = false;
        private bool itemConfigNotificationFlag = false;
        private bool externalCarConfigNotification = false;
        int count = 0;

        public SingletonObjectTestFixture()
        {
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region Singleton object test fixture

        [TestMethod]
        public void CreateObjectTest()
        {
            CarFactory factory = new CarFactory();
            Car obj1 = factory.Create("MyCar");
            Assert.IsNotNull(obj1); 
        }

        [TestMethod]
        public void VerifyObjectCreatedOnlyOnce()
        {
            CarFactory factory = new CarFactory();
            Car obj1 = factory.Create("MyCar");
            Car obj2 = factory.Create("MyCar");
            Assert.IsNotNull(obj1);
            Assert.IsNotNull(obj2);
            Assert.IsTrue(object.ReferenceEquals(obj1,obj2));    
        }

        [TestMethod]
        public void VerifyObjectState()
        {
            CarFactory factory = new CarFactory();
            Car obj1 = factory.Create("MyCar");
            obj1.IncreaseSpeed();
            Car obj2 = factory.Create("MyCar");
            obj2.IncreaseSpeed();
            Assert.IsTrue(obj1.Speed ==  obj2.Speed);
        }
#endregion

        #region file configuration watcher tests
        [TestMethod]
        public void VerifyNotificationHasOccured()
        {
            itemConfigNotificationFlag  = false;
            carConfigNotificationFlag = false;
            count = 0;
            SystemConfigurationSource source = new SystemConfigurationSource();
            CarSettings setting = new CarSettings(new CarData("MyCar", "Brand1", "2005", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting);
            source.AddSectionChangeHandler("carConfiguration", new ConfigurationChangedEventHandler(OnConfigurationChanged));
            CarSettings  settings = (CarSettings)source.GetSection("carConfiguration"); 
            CarFactory factory = new CarFactory();
            CarSettings setting1 = new CarSettings(new CarData("MyCar", "Brand1", "1905", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting1);
            System.Threading.Thread.Sleep(20000);
            Car myCar = factory.Create("MyCar");
            Assert.IsTrue(carConfigNotificationFlag);
        }

        [TestMethod]
        public void VerifyNotificationOccuredOnlyOnce()
        {
            itemConfigNotificationFlag = false;
            carConfigNotificationFlag = false;
            count = 0;
            SystemConfigurationSource source = new SystemConfigurationSource();
            CarSettings setting = new CarSettings(new CarData("MyCar", "Brand1", "2005", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting);
            source.AddSectionChangeHandler("carConfiguration", new ConfigurationChangedEventHandler(OnConfigurationChanged));
            CarSettings settings = (CarSettings)source.GetSection("carConfiguration");
            settings = (CarSettings)source.GetSection("carConfiguration");
            settings = (CarSettings)source.GetSection("carConfiguration");
            CarFactory factory = new CarFactory();
            CarSettings setting1 = new CarSettings(new CarData("MyCar", "Brand1", "1905", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting1);
            System.Threading.Thread.Sleep(16000);
            Car myCar = factory.Create("MyCar");
            Assert.IsTrue(carConfigNotificationFlag);
            Assert.IsTrue(count == 1);
        }

        [TestMethod]
        public void VerifyNotificationCancelledWhenUnregistered()
        {
            itemConfigNotificationFlag = false;
            carConfigNotificationFlag = false;
            count = 0;
            ConfigurationChangedEventHandler hanlder = new ConfigurationChangedEventHandler(OnConfigurationChanged);  
            SystemConfigurationSource source = new SystemConfigurationSource();
            CarSettings setting = new CarSettings(new CarData("MyCar", "Brand1", "2005", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting);
            source.AddSectionChangeHandler("carConfiguration", hanlder);
            CarSettings settings = (CarSettings)source.GetSection("carConfiguration");
            source.RemoveSectionChangeHandler("carConfiguration", hanlder);    
            CarFactory factory = new CarFactory();
            CarSettings setting1 = new CarSettings(new CarData("MyCar", "Brand1", "1905", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting1);
            System.Threading.Thread.Sleep(16000);
            Car myCar = factory.Create("MyCar");
            Assert.IsFalse(carConfigNotificationFlag);
            Assert.IsTrue(count == 0);
        }

        [TestMethod]
        public void VerifyNotificationOccursChangedSection()
        {
            itemConfigNotificationFlag = false;
            carConfigNotificationFlag = false;
            count = 0;
            ConfigurationChangedEventHandler hanlder = new ConfigurationChangedEventHandler(OnConfigurationChanged);
            SystemConfigurationSource source = new SystemConfigurationSource();
            CarSettings setting = new CarSettings(new CarData("MyCar", "Brand1", "2005", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting);
            source.AddSectionChangeHandler("carConfiguration", hanlder);
            source.AddSectionChangeHandler("itemConfiguration", hanlder);
            CarSettings settings = (CarSettings)source.GetSection("carConfiguration");
            CarFactory factory = new CarFactory();
            CarSettings setting1 = new CarSettings(new CarData("MyCar", "Brand1", "1905", new CarPartsData("CarParts", "Steel")));
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting1);
            System.Threading.Thread.Sleep(16000);
            Car myCar = factory.Create("MyCar");
            Assert.IsTrue(carConfigNotificationFlag);
        }

        [TestMethod]
        public void VerifyNotificationOccursAllSection()
        {
            itemConfigNotificationFlag = false;
            carConfigNotificationFlag = false;
            count = 0;
            ConfigurationChangedEventHandler hanlder = new ConfigurationChangedEventHandler(OnConfigurationChanged);
            SystemConfigurationSource source = new SystemConfigurationSource();
            CarSettings setting = new CarSettings(new CarData("MyCar", "Brand1", "2005", new CarPartsData("CarParts", "Steel")));
            ItemSettings itemSetting = new ItemSettings();
            itemSetting.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            itemSetting.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 100, 10, "1.0.0.0"));
            itemSetting.DefaultItem = "Item1";
    
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting);
            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "itemConfiguration",itemSetting);
            source.AddSectionChangeHandler("carConfiguration", hanlder);
            source.AddSectionChangeHandler("itemConfiguration", hanlder);
            CarSettings settings = (CarSettings)source.GetSection("carConfiguration");
            source.GetSection("itemConfiguration");
            CarFactory factory = new CarFactory();
            CarSettings setting1 = new CarSettings(new CarData("MyCar", "Brand1", "1905", new CarPartsData("CarParts", "Steel")));
            ItemSettings itemSetting1 = new ItemSettings();
            itemSetting1.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            itemSetting1.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 100, 10, "2.0.0.0"));
            itemSetting1.DefaultItem = "Item1";

            source.Save(ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None).FilePath, "carConfiguration", setting1);
            System.Threading.Thread.Sleep(16000);
            Car myCar = factory.Create("MyCar");
            Assert.IsTrue(carConfigNotificationFlag);
            Assert.IsTrue(itemConfigNotificationFlag);
            Assert.IsTrue(count == 2);
        }


        private void OnConfigurationChanged(object sender, ConfigurationChangedEventArgs e)
        {
            if (e.SectionName == "carConfiguration")
            {
                carConfigNotificationFlag = true;
            }
            else if (e.SectionName == "itemConfiguration")
            {
                itemConfigNotificationFlag = true;
            }
            else if (e.SectionName == "externalcarConfiguration")
            {
                externalCarConfigNotification = true;
            }

            count++;
        }

        private void UpdateConfigFile(string fileName,string data)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(fileName);
            XmlNodeList nodeList = doc.SelectNodes("/externalcarConfiguration");
            foreach (XmlNode node in nodeList)
            {
                foreach (XmlAttribute xatt in node.FirstChild.Attributes)
                {
                    if (xatt.Name == "model" )
                    {
                        xatt.Value = data;
                    }
                }
            }
            doc.Save(fileName);
        }
        #endregion
    }
}
