using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using System.Collections.Specialized;

namespace CryptoFuncTests.CryptoProviderTestFixtures
{
    /// <summary>
    /// Summary description for CustomCryptoProviderTest
    /// </summary>
    [TestClass]
    public class CustomCryptoProviderTest
    {
        public CustomCryptoProviderTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestOfCustomCryptoProviderUsingDictionarySource()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            CustomSymmetricCryptoProviderData data = new CustomSymmetricCryptoProviderData("test",typeof(TestCustomCrypto));
            data.SetAttributeValue("key1", "value1");
            data.SetAttributeValue("key2", "value2");

            setting.SymmetricCryptoProviders.Add(data);
            DictSource.Add("securityCryptographyConfiguration", setting);
            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(DictSource);
            ISymmetricCryptoProvider symmetricProvider = factory.Create("test");
            Assert.IsNotNull(symmetricProvider);
            Assert.IsTrue(symmetricProvider is TestCustomCrypto);
            Assert.AreEqual(((TestCustomCrypto)symmetricProvider).ObjCol.Count, 2);
            Assert.AreEqual(((TestCustomCrypto)symmetricProvider).ObjCol["key1"], "value1");
            Assert.AreEqual(((TestCustomCrypto)symmetricProvider).ObjCol["key2"], "value2");

        }

        [TestMethod]
        public void TestOfCustomCryptoProviderUsingSystemSource()
        {

            SymmetricCryptoProviderFactory factory = new SymmetricCryptoProviderFactory(new SystemConfigurationSource());
            ISymmetricCryptoProvider symmetricProvider = factory.Create("testCustom");
            Assert.IsNotNull(symmetricProvider);
            Assert.IsTrue(symmetricProvider is TestCustomCrypto);
            Assert.AreEqual(((TestCustomCrypto)symmetricProvider).ObjCol["key1"], "value1");
            Assert.AreEqual(((TestCustomCrypto)symmetricProvider).ObjCol["key2"], "value2");
        }

        

       

    }

# region helper Class 
   
    [ConfigurationElementType(typeof(CustomSymmetricCryptoProviderData))]
    public class TestCustomCrypto:ISymmetricCryptoProvider

    {
        public NameValueCollection ObjCol;
        public TestCustomCrypto(NameValueCollection obj)
        {
            this.ObjCol = obj;
        }


        public byte[] Encrypt(byte[] plaintext)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public byte[] Decrypt(byte[] ciphertext)
        {
            throw new Exception("The method or operation is not implemented.");
        }

       
    }
# endregion
}
