using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using System.Collections.Specialized;

namespace CryptoFuncTests.CryptoProviderTestFixtures
{
    /// <summary>
    /// Summary description for CustomCryptoProviderTest
    /// </summary>
    [TestClass]
    public class CustomHashProviderTest
    {
        public CustomHashProviderTest()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestOfCustomHashProviderUsingDictionarySource()
        {
            DictionaryConfigurationSource DictSource = new DictionaryConfigurationSource();
            CryptographySettings setting = new CryptographySettings();
            CustomHashProviderData data = new CustomHashProviderData("test",typeof(TestCustomHash));
            data.SetAttributeValue("key1", "value1");
            data.SetAttributeValue("key2", "value2");
            setting.HashProviders.Add(data);
            DictSource.Add("securityCryptographyConfiguration", setting);
            HashProviderFactory factory = new HashProviderFactory(DictSource);
            IHashProvider symmetricProvider = factory.Create("test");
            Assert.IsNotNull(symmetricProvider);
            Assert.IsTrue(symmetricProvider is TestCustomHash);
            Assert.AreEqual(((TestCustomHash)symmetricProvider).ObjCol.Count, 2);
            Assert.AreEqual(((TestCustomHash)symmetricProvider).ObjCol["key1"], "value1");
            Assert.AreEqual(((TestCustomHash)symmetricProvider).ObjCol["key2"], "value2");

        }

        [TestMethod]
        public void TestOfHashCryptoProviderUsingSystemSource()
        {

            HashProviderFactory factory = new HashProviderFactory(new SystemConfigurationSource());
            IHashProvider provider = factory.Create("testcustomHash");

            Assert.IsNotNull(provider);
            Assert.IsTrue(provider is TestCustomHash);
        }



    }

# region helper Class 
   
    [ConfigurationElementType(typeof(CustomHashProviderData))]
    public class TestCustomHash:IHashProvider

    {
        public NameValueCollection ObjCol;
        public TestCustomHash(NameValueCollection obj)
        {
            this.ObjCol = obj;

        }


        
        #region IHashProvider Members

        public byte[] CreateHash(byte[] plaintext)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool CompareHash(byte[] plaintext, byte[] hashedtext)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
# endregion
}
