using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Diagnostics;
using System.Management;
using System.Messaging;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;

namespace CryptoFuncTests
{
    /// <summary>
    /// Summary description for TestingInstrumentation
    /// </summary>
    [TestClass]
    public class InstrumentationTestFixture
    {
        string CategoryName = "Enterprise Library Cryptography Counters";
        string loggedText = "";
        bool logged = false;
        ManagementScope scope = null;

        public InstrumentationTestFixture()
        {
            scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;

        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion


        [TestMethod]
        public void HashOperationCompletedEventTest()
        {
           PerformanceCounterInstanceName pf = new PerformanceCounterInstanceName(AppDomain.CurrentDomain.FriendlyName, "HashAlgorithm1");
           using (PerformanceCounter perf = new PerformanceCounter(CategoryName, @"Hash Operations/sec", pf.ToString()))
           {
               HashProviderFactory fact = new HashProviderFactory(new SystemConfigurationSource());
               IHashProvider prov = fact.Create("HashAlgorithm1");
               long initialCount = perf.RawValue;
               prov.CreateHash(new byte[] { 1, 23 });
               long finalCount = perf.RawValue;  
               Assert.AreEqual((long)1, finalCount - initialCount);
           }
        }

        [TestMethod]
        public void HashComparisionsEventTest()
        {
            PerformanceCounterInstanceName pf = new PerformanceCounterInstanceName(AppDomain.CurrentDomain.FriendlyName, "HashAlgorithm1");
            using (PerformanceCounter perf = new PerformanceCounter(CategoryName, @"Hash Comparisons/sec", pf.ToString()))
            {
                HashProviderFactory fact = new HashProviderFactory(new SystemConfigurationSource());
                IHashProvider prov = fact.Create("HashAlgorithm1");
                long initialCount = perf.RawValue;
                byte[] dig = prov.CreateHash(new byte[] { 1, 23 });
                prov.CompareHash(new byte[] { 1, 23 }, dig);
                long finalCount = perf.RawValue;
                Assert.AreEqual((long)1, finalCount - initialCount);
            }
        }

        [TestMethod]
        public void HashDigestMismatchEventTest()
        {
            long initialCount, initialCount1, finalCount, finalCount1;
            ManagementEventWatcher watcher = null;
            PerformanceCounter perf = null, perf1 = null;
            try
            {
                PerformanceCounterInstanceName pf = new PerformanceCounterInstanceName(AppDomain.CurrentDomain.FriendlyName, "HashAlgorithm1");
                perf = new PerformanceCounter(CategoryName, @"Hash Comparisons/sec", pf.ToString());
                perf1 = new PerformanceCounter(CategoryName, @"Hash Mismatches/sec", pf.ToString());

                HashProviderFactory fact = new HashProviderFactory(new SystemConfigurationSource());
                IHashProvider prov = fact.Create("HashAlgorithm1");
                EventQuery eq = new EventQuery("SELECT * FROM HashMismatchDetectedEvent");
                watcher = new ManagementEventWatcher(scope, eq);
                watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
                watcher.Start();
                logged = false;
                loggedText = "";

                initialCount = perf.RawValue;
                initialCount1 = perf1.RawValue;

                byte[] dig = prov.CreateHash(new byte[] { 1, 23 });
                prov.CompareHash(new byte[] { 1, 24 }, dig);

                finalCount = perf.RawValue;
                finalCount1 = perf1.RawValue;
            }
            finally
            {
                if(watcher != null) 
                    watcher.Stop();
                perf.Dispose();
                perf1.Dispose();
            }
            
            Assert.AreEqual((long)1, finalCount - initialCount);
            Assert.AreEqual((long)1, finalCount1 - initialCount1);

            Assert.IsTrue(logged);
            Assert.IsTrue(loggedText.Contains("HashMismatchDetectedEvent"));    
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            logged = true;
            loggedText = args.NewEvent.GetText(TextFormat.Mof);
        }


    }
}
