using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Security.Cryptography;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Security.Cryptography;    
  
namespace KeyFileGenerator
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                string filePath = AppDomain.CurrentDomain.BaseDirectory;
                filePath = filePath.Substring(0, filePath.IndexOf("Cryptography Functional Tests"));
                filePath += "Cryptography Functional Tests\\CryptoFuncTests\\KeyFiles"; 
                //Symmetric keys
                GenerateSymmetricKeyFile(Path.Combine(filePath,"RC2CryptoServiceProvider.key"), typeof(RC2CryptoServiceProvider), DataProtectionScope.CurrentUser, 0);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"DesCryptoserviceProvider.key"), typeof(DESCryptoServiceProvider), DataProtectionScope.CurrentUser, 0);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"Rijndeal.key"), typeof(RijndaelManaged), DataProtectionScope.CurrentUser,0);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"Rijndael128.key"), typeof(RijndaelManaged), DataProtectionScope.CurrentUser, 128);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"Rijndael192.key"), typeof(RijndaelManaged), DataProtectionScope.CurrentUser, 192);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"Rijndael256.key"), typeof(RijndaelManaged), DataProtectionScope.CurrentUser, 256);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"tripleDesCryptoServiceProvider.key"), typeof(TripleDESCryptoServiceProvider), DataProtectionScope.CurrentUser, 0);
                GenerateSymmetricKeyFile(Path.Combine(filePath,"Rijndael256MachineMode.key"), typeof(RijndaelManaged), DataProtectionScope.LocalMachine, 1256);

                //Hash Keys
                GenerateHashKeyFile(Path.Combine(filePath,"hmac1-1.key"), typeof(HMACSHA1), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HMacMd5.key"), typeof(HMACMD5), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HMACRIPEMD160.key"), typeof(HMACRIPEMD160), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"Hmac1.key"), typeof(HMACSHA1), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HMAC256.key"), typeof(HMACSHA256), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HMAC300.key"), typeof(HMACSHA384), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HMAC512.key"), typeof(HMACSHA512), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"MacTripleDes.key"), typeof(MACTripleDES), DataProtectionScope.CurrentUser);
                GenerateHashKeyFile(Path.Combine(filePath,"HmacSha512MachineMode.key"), typeof(HMACSHA512), DataProtectionScope.LocalMachine);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(Environment.NewLine);
                Console.WriteLine(e.StackTrace);   
            }
        }

        public static void GenerateSymmetricKeyFile(string fileName, Type algorithmType, DataProtectionScope scope,int keySize)
        {
            ProtectedKey symmetricKey = null;
            switch (keySize)
            {
                default:
                    symmetricKey = KeyManager.GenerateSymmetricKey(algorithmType, scope);
                    break;
                case 128:
                case 192:
                case 256:
                    SymmetricAlgorithm algorithm = (SymmetricAlgorithm) Activator.CreateInstance(algorithmType);
                    algorithm.KeySize = keySize;
                    algorithm.GenerateKey();
                    symmetricKey = ProtectedKey.CreateFromPlaintextKey(algorithm.Key,scope);
                    break;
            }
            using (FileStream keyStream = new FileStream(fileName, FileMode.Create))
            {
                KeyManager.Write(keyStream, symmetricKey);
            }
        }

        public static void GenerateHashKeyFile(string fileName, Type algorithmType, DataProtectionScope scope)
        {
            ProtectedKey hashKey = null;
            hashKey = KeyManager.GenerateKeyedHashKey(algorithmType, scope);
            using (FileStream keyStream = new FileStream(fileName, FileMode.Create))
            {
                KeyManager.Write(keyStream, hashKey);
            }
        }

    }
}
