using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Transactions;
using System.Data.SqlClient;
using System.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Data;
using System.Data.Common;

namespace DataFuncTests.SqlDatabaseFixtures
{
    /// <summary>
    /// Summary description for DTCFixture
    /// </summary>
    [TestClass]
    public class DTCFixture
    {
        string connStr;
        public DTCFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize()
        {
            connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Delete  from DTCTable", conn);
            cmd.ExecuteNonQuery();
            conn.Close();
        }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion


        [TestMethod]
        public void TransactionScopeFailsRollbackTest()
        {
            using (SqlConnection conn = new SqlConnection())
            {
                conn.ConnectionString = connStr;
                conn.Open();
                SqlTransaction trans = conn.BeginTransaction();
                SqlCommand cmRowCont = null; ;
                int noofrows = 0;
                try
                {
                    SqlCommand cmd = new SqlCommand("insert into DTCTable Values(1,'Narayan')", conn, trans);
                    cmd.ExecuteNonQuery();
                    cmRowCont = new SqlCommand("select count(eno) from DTCTable", conn, trans);
                    noofrows = (int)cmRowCont.ExecuteScalar();
                    using (TransactionScope ts = new TransactionScope(TransactionScopeOption.RequiresNew))
                    {
                        //generating an exception
                        Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
                        SqlConnection conn1 = new SqlConnection();
                        conn1.ConnectionString = connStr;
                        //primary key violation 
                        SqlCommand cmd1 = new SqlCommand("insert into DTCTable values(1,'Narayan')", conn1);
                        db.ExecuteNonQuery(cmd1);
                        ts.Complete();
                    }

                    trans.Commit();
                }
                catch (Exception )
                {
                    trans.Rollback();
                    int noofrowsafter = (int)cmRowCont.ExecuteScalar();
                    Assert.AreEqual(noofrows-1, noofrowsafter);
                }
                finally
                {
                    conn.Close();
                }

            }
        }

        [TestMethod]
        public void DAABTransactionWithDTCTest()
        {
            using (TransactionScope ts = new TransactionScope(TransactionScopeOption.RequiresNew))
            {
                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmd = new SqlCommand("insert into DTCTable Values(1,'Narayan')", conn);
                    cmd.ExecuteNonQuery();
                }
                // using DAAB with transaction
                Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
                using (DbConnection connection = db.CreateConnection())
                {
                    connection.Open();
                    DbTransaction transaction = connection.BeginTransaction();
                    db.ExecuteNonQuery(transaction, CommandType.Text, "insert into DTCTable Values(2,'Nandan')");
                    transaction.Commit();  

                }
                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmRowCont = new SqlCommand("select count(eno) from DTCTable", conn);
                    int noofrows = (int)cmRowCont.ExecuteScalar();
                    Assert.AreEqual(noofrows, 2);
                }
                ts.Complete();
            }
        }

        [TestMethod]
        public void DAABWithoutTransactionWithDTCTest()
        {
            using (TransactionScope ts = new TransactionScope(TransactionScopeOption.RequiresNew))
            {
                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmd = new SqlCommand("insert into DTCTable Values(1,'Narayan')", conn);
                    cmd.ExecuteNonQuery();
                }
                // using DAAB with transaction
                Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
                using (SqlConnection conn1 = new SqlConnection())
                {


                    conn1.ConnectionString = connStr;
                    SqlCommand cmd = new SqlCommand("insert into DTCTable values(2,'Nandan')", conn1);
                    db.ExecuteNonQuery(cmd);

                }

                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmRowCont = new SqlCommand("select count(eno) from DTCTable", conn);
                    int noofrows = (int)cmRowCont.ExecuteScalar();
                    Assert.AreEqual(noofrows, 2);
                    conn.Close();
                }
                ts.Complete();

            }


        }

        [TestMethod]
        public void MultipleTransactionScopeTest()
        {
            using (TransactionScope ts = new TransactionScope(TransactionScopeOption.RequiresNew))
            {
                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmd = new SqlCommand("insert into DTCTable Values(1,'Narayan')", conn);
                    cmd.ExecuteNonQuery();
                }
                try
                {
                    using (TransactionScope ts1 = new TransactionScope(TransactionScopeOption.Suppress))
                    {
                        Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
                        SqlConnection conn1 = new SqlConnection();
                        conn1.ConnectionString = connStr;
                        SqlCommand cmd = new SqlCommand("insert into DTCTable values(1,'Nandan')", conn1);
                        db.ExecuteNonQuery(cmd);
                        ts1.Complete();
                    }
                }
                catch (Exception)
                {

                }

                using (SqlConnection conn = new SqlConnection())
                {
                    conn.ConnectionString = connStr;
                    conn.Open();
                    SqlCommand cmRowCont = new SqlCommand("select count(eno) from DTCTable", conn);
                    int noofrows = (int)cmRowCont.ExecuteScalar();
                    Assert.AreEqual(noofrows, 1);
                    conn.Close();
                }
                ts.Complete();


            }

        }



    }
}
