using System;
//using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Data.Sql;
using System.Xml;
using System.Data.Common;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Configuration;

namespace DataFuncTests.GenericDatabaseFixturesFixtures
{
    /// <summary>
    /// Tests ExecuteDataSet Method of the DataSet Object
    /// </summary>
    [TestClass]
    public class ExecuteDataSetTestFixture
    {
        String connStr;
        String itemsXMLfile;
        DataSet dsExpectedResult = new DataSet();
        public ExecuteDataSetTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                   "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            cmd.ExecuteNonQuery();
            conn.Close();
            itemsXMLfile = "Items.xml";
            dsExpectedResult = new DataSet();
            dsExpectedResult.ReadXml(itemsXMLfile);

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            dsExpectedResult.Clear();  
            dsExpectedResult.Dispose();  
        }

        #region "DB Command"
        [TestMethod]
        public void DSDBCommandQueryTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "Select ItemDescription from Items order by ItemID";
            DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
            int index;
            using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper))
            {
                int count = dsActualResult.Tables[0].Rows.Count;
                for (index = 0; index < count; index++)
                {
                    Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index][0].ToString().Trim());
                }

            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DSDBCommandSPNoParmTest()
        {
            int index;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string spName = "ItemsGet";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(spName);
            using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper))
            {
                int count = dsActualResult.Tables[0].Rows.Count;
                for (index = 0; index < count; index++)
                {
                    Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                }
            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);

        }

        [TestMethod]
        public void DSDBCommandSPOutParmTest()
        {

            int index;


            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");

            DbCommand dbCommandWrapper = db.GetStoredProcCommand("ItemPriceGetById");
            this.CreateCommandParameter(db,dbCommandWrapper, "@ID", DbType.Int32, ParameterDirection.Input, 1);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Price", DbType.Currency, ParameterDirection.Output, 50);
            using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper))
            {
                int count = dsActualResult.Tables[0].Rows.Count;
                for (index = 0; index < count; index++)
                {
                    Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                }
            }

            Assert.AreEqual(Convert.ToDouble(38.95), Convert.ToDouble(dbCommandWrapper.Parameters["@Price"].Value));
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        public void DSDBCommandSPUpdateParmTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "ItemQuantityUpdateById";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
            this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 55);
            using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper))
            {
                int count = dsActualResult.Tables[0].Rows.Count;
                for (int index = 0; index < count; index++)
                {
                    if (Convert.ToInt16(dsActualResult.Tables[0].Rows[index]["ItemId"]) == 1)
                    {
                        Assert.AreEqual(55, Convert.ToInt32(dsActualResult.Tables[0].Rows[index]["QtyInHand"]));
                    }

                }
               
            }
            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);
        }

        [TestMethod]
        [ExpectedException(typeof(OleDbException))]
        public void DSDBCommandNoSPPresentTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            string sqlCommand = "UpdateItem1";
            DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
            using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper))
                { }
            

            Assert.AreEqual(ConnectionState.Closed, dbCommandWrapper.Connection.State);

        }

        #endregion

        #region "Db Command, Transaction"
        [TestMethod]
        public void DSDBCommandTransSqlNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand= 88 where ItemId = 2  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("select * from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);

        }

        [TestMethod]
        public void DSDBCommandTransSqlCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 45 where ItemId = 1  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Connection = connection;
                dbCommandWrapper.Transaction = transaction;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;

                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Commit();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(45, (int) dr["QtyInHand"]);
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSDBCmdTransSqlRollbackTest()
        {


            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "begin update Items set QtyInHand = 67 where ItemId = 2  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();

        }

        [TestMethod]
        public void DSDBCmdTransSqlCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "begin update Items set QtyInHand = 111 where ItemId = 1  select * from Items end";
                DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;

                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;

                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "begin update Items set QtyInHand = 55 where ItemId = 2 select * from Items end";
                dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {

                      Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Rollback();
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(111), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
           dr.Close();
            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
           dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }

            dr.Close();
            conn.Close();

        }
        [TestMethod]
        public void DSDBCmdTransSqlRollbackOnErrorTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            DbTransaction transaction = null;
            try
            {
                using (DbConnection connection = db.CreateConnection())
                {
                    connection.Open();
                    transaction = connection.BeginTransaction();
                    string sqlCommand = "begin update Items set QtyInHand = 45 where ItemId = 2  select * from Items where ItemId = 'Test' end";
                    DbCommand dbCommandWrapper = db.GetSqlStringCommand(sqlCommand);
                    dbCommandWrapper.Transaction = transaction;
                    dbCommandWrapper.Connection = connection;
                    using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                    {
                        int count = dsActualResult.Tables[0].Rows.Count;
                        for (int i = 0; i < count; i++)
                        {
                           Assert.AreEqual(dsExpectedResult.Tables[0].Rows[i][0].ToString().Trim(), dsActualResult.Tables[0].Rows[i]["ItemDescription"].ToString().Trim());
                        }
                    }
                    transaction.Commit();
                }
            }
            catch
            {
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();

            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }



        [TestMethod]
        public void DSDBCmdTransSPNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 67);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int i = 0; i < count; i++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[i][0].ToString().Trim(), dsActualResult.Tables[0].Rows[i]["ItemDescription"].ToString().Trim());
                    }
                }
            }
            //Assert.AreEqual(ConnectionState.Closed, db.Connection.State);
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
            
        }

        [TestMethod]
        public void DSDBCmdTransSPCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 88);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection; 
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int i = 0; i < count; i++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[i][0].ToString().Trim(), dsActualResult.Tables[0].Rows[i]["ItemDescription"].ToString().Trim());
                    }
                }

                transaction.Commit();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(88), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSDBCmdTransSPRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 65);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                       Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }

                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSDBCmdTransSPCommitRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerData = new StringBuilder();
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();

                string sqlCommand = "ItemQuantityUpdateById";
                DbCommand dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 1);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 44);
                dbCommandWrapper.Transaction = transaction;
                dbCommandWrapper.Connection = connection;
                using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;

                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Commit();
                transaction = connection.BeginTransaction();
                sqlCommand = "ItemQuantityUpdateById";
                dbCommandWrapper = db.GetStoredProcCommand(sqlCommand);
                this.CreateCommandParameter(db,dbCommandWrapper, "@ItemId", DbType.Int16, ParameterDirection.Input, 2);
                this.CreateCommandParameter(db,dbCommandWrapper, "@Quantity", DbType.Int16, ParameterDirection.Input, 72);
               using (DataSet dsActualResult = db.ExecuteDataSet(dbCommandWrapper, transaction))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int i = 0; i < count; i++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[i][0].ToString().Trim(), dsActualResult.Tables[0].Rows[i]["ItemDescription"].ToString().Trim());
                    }
                }
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(44), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();

            cmd = new SqlCommand("Select QtyInHand from Items where ItemId = 2", conn);
            dr = cmd.ExecuteReader();

            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }

            dr.Close();
            conn.Close();


        }
        #endregion

        #region "ExecuteDataSet(Commandtype)"

        [TestMethod]

        public void ExecuteDataSetCommandtypeSqlTest()
        {

            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string sqlText = "select ItemDescription From Items ";
            using (DataSet dsActualResult = db.ExecuteDataSet(CommandType.Text, sqlText))
            {
                int count = dsActualResult.Tables[0].Rows.Count;
                for (int index = 0; index < count; index++)
                {
                    Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                }

            }
        }

        [TestMethod]
        public void ExecuteDataSetCommandtypeSpTest()
        {
            DataSet dsExpectedData = new DataSet();
            dsExpectedData.ReadXml(itemsXMLfile);
            int count = 0;
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "ItemsGet";
            using (DataSet dsActualResult = db.ExecuteDataSet(CommandType.StoredProcedure, spName))
            {
                count = dsActualResult.Tables[0].Rows.Count;
                for (int index = 0; index < count; index++)
                {
                    Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                }
            }
        }

        [TestMethod]
        [ExpectedException(typeof(OleDbException))]

        public void ExecuteDataSetNotExistingSPTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            StringBuilder readerDataActual = new StringBuilder();
            string spName = "something";
            using (DataSet dsActualResult = db.ExecuteDataSet(CommandType.StoredProcedure, spName))
            {
            }

        }
        #endregion

        #region "Transaction, Command Type, Command Name"

        [TestMethod]
        public void DSTransQueryTest()
        {

            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.Text, "begin update Items set QtyInHand = 175 where ItemId = 1 select * from Items order by ItemId end "))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }


                }

                transaction.Commit();

            }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(175), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSTransQueryNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.Text, "begin update Items set QtyInHand = 67 where ItemId = 2 select * from Items order by ItemId end "))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }
                }
             }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSTransQueryRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.Text, "begin update Items set QtyInHand = 18 where ItemId = 2 select * from Items end "))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int i = 0; i < count; i++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[i][0].ToString().Trim(), dsActualResult.Tables[0].Rows[i]["ItemDescription"].ToString().Trim());
                    }

                }
                transaction.Rollback();
            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where ItemId=2", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
               Assert.AreEqual(Convert.ToDouble(95), Convert.ToDouble(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]

        public void DSTransSpCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.StoredProcedure, "ItemQuantityUpdateById"))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                   }
                }
               transaction.Commit();
            }

            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyInHand from Items where itemid=1", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(50, Convert.ToInt32(dr["QtyInHand"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSTransSpNoCommitTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease"))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }

                }

           }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }

        [TestMethod]
        public void DSTransSpRollbackTest()
        {
            Database db = DatabaseFactory.CreateDatabase("GenericSQLTest");
            using (DbConnection connection = db.CreateConnection())
            {
                connection.Open();
                DbTransaction transaction = connection.BeginTransaction();
                using (DataSet dsActualResult = db.ExecuteDataSet(transaction, CommandType.StoredProcedure, "ItemQuantityIncrease"))
                {
                    int count = dsActualResult.Tables[0].Rows.Count;
                    for (int index = 0; index < count; index++)
                    {
                        Assert.AreEqual(dsExpectedResult.Tables[0].Rows[index][0].ToString().Trim(), dsActualResult.Tables[0].Rows[index]["ItemDescription"].ToString().Trim());
                    }

                }

                transaction.Rollback();

            }
            SqlConnection conn = new SqlConnection();
            StringBuilder readerDataExpected = new StringBuilder();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("Select QtyRequired from Items", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            while (dr.Read())
            {
                Assert.AreEqual(100, Convert.ToInt32(dr["QtyRequired"].ToString()));
            }
            dr.Close();
            conn.Close();
        }
        #endregion

        # region Helper Method
        private void CreateCommandParameter(Database db, DbCommand comm, string Parametername, DbType dbType, ParameterDirection dir, object value)
        {

            if (dir == ParameterDirection.Input)
            {
                db.AddInParameter(comm, Parametername, dbType, value);
            }
            else if (dir == ParameterDirection.Output)
            {
                db.AddOutParameter(comm, Parametername, dbType, (int)value);
            }

        }
        # endregion

    }
}
