using System;
//using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using Microsoft.Practices.EnterpriseLibrary.Data.Oracle;
using System.Data;
using System.Data.Common;
using System.Text;
using System.Data.SqlClient;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Configuration;

namespace DataFuncTests.SqlDatabaseFixtures
{
    /// <summary>
    /// Summary description for GetdataadapterTestFixture
    /// </summary>
    [TestClass]
    public class GetDataAdapterTestFixture
    {
        string itemsXMLfile;
        string connStr;
        public GetDataAdapterTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }
        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            itemsXMLfile = "items.xml";
           connStr = ConfigurationManager.ConnectionStrings["DataSQLTest"].ConnectionString;
            SqlConnection conn = new SqlConnection();
            conn.ConnectionString = connStr;
            conn.Open();
            SqlCommand cmd = new SqlCommand("begin Delete from Items where itemId > 3 " +
                                                   "Update Items set ItemDescription='Digital Image Pro',QtyInHand = 25, QtyRequired=100 where ItemId = 1 " +
                                                   "Update Items set ItemDescription='Excel 2003',QtyInHand = 95, QtyRequired=100 where ItemId = 2 " +
                                                   "Update Items set ItemDescription='Infopath',QtyInHand = 34, QtyRequired=100 where ItemId = 3 " +
                                                   "delete from CustomersOrders where CustomerId > 3 " +
                                                   "Update CustomersOrders set QtyOrdered=100 " +
                                            "end", conn);
            SqlDataReader dr = cmd.ExecuteReader();
            dr.Close();
            conn.Close();
        }
        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        // (" DbDataAdapter.GetDataAdapter(UpdateBehavior) has protected acess")]
        [TestMethod]
        public void GetDataAdapterTransactionalTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            DataSet itemsDataSet = new DataSet();
            DbCommand command = db.GetStoredProcCommand("ItemsGet");
            //using (DbDataAdapter adapter = db.GetDataAdapter(UpdateBehavior.Transactional))
            using (DbDataAdapter adapter = db.GetDataAdapter())
            {
                using (DbConnection conn = db.CreateConnection())
                {
                    conn.Open();
                    DbTransaction trans = conn.BeginTransaction() as DbTransaction;
                    command.Connection = conn;
                    ((IDbDataAdapter)adapter).SelectCommand = command;//.Command;
                    command.Transaction = trans;
                    adapter.TableMappings.Add("Table", "Items");
                    ((IDbDataAdapter)adapter).Fill(itemsDataSet);
                    DataTable products = itemsDataSet.Tables["Items"];
                    products.Rows.Add(new object[] { 4, "New Row Added", 100, 100, 100 });
                    products.Rows[3].Delete();
                    DbCommand insertCommandWrapper = db.GetStoredProcCommandWithSourceColumns("AddItem", "ItemID", "ItemDescription", "Price", "QtyinHand", "QtyRequired");
                    DbCommand deleteCommandWrapper = db.GetStoredProcCommandWithSourceColumns("deleteItem", "ItemID");
                    DbCommand updateCommandWrapper = db.GetStoredProcCommandWithSourceColumns("UpdateItem", "ItemID", "ItemDescription");
                    insertCommandWrapper.Connection = conn;
                    insertCommandWrapper.Transaction = trans;
                    deleteCommandWrapper.Connection = conn;
                    deleteCommandWrapper.Transaction = trans;
                    updateCommandWrapper.Connection = conn;
                    updateCommandWrapper.Transaction = trans;
                    ((IDbDataAdapter)adapter).InsertCommand = insertCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).DeleteCommand = deleteCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).UpdateCommand = updateCommandWrapper;//.Command;
                    adapter.Update(itemsDataSet);
                    trans.Commit();
                }
            }
            DataSet da = new DataSet();
            da.ReadXml(itemsXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            Assert.AreEqual(da.Tables[0].Rows.Count, itemsDataSet.Tables[0].Rows.Count);
            for (index = 0; index < da.Tables[0].Rows.Count; index++)
            {
                Assert.AreEqual(da.Tables[0].Rows[index][0].ToString().Trim(), itemsDataSet.Tables[0].Rows[index][1].ToString().Trim());
            }
        }

        //(" DbDataAdapter.GetDataAdapter(UpdateBehavior) has protected acess")]
        [TestMethod]
        public void GetDataAdapterContinueTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            DataSet itemsDataSet = new DataSet();
            DbCommand command = db.GetStoredProcCommand("ItemsGet");
            //DbDataAdapter adapter = db.GetDataAdapter(UpdateBehavior.Continue)
            using (DbDataAdapter adapter = db.GetDataAdapter())
            {
                using (DbConnection conn = db.CreateConnection())
                {
                    conn.Open();
                    DbTransaction trans = conn.BeginTransaction();
                    command.Connection = conn;
                    ((IDbDataAdapter)adapter).SelectCommand = command;//.Command;
                    command.Transaction = trans;
                    adapter.TableMappings.Add("Table", "Items");
                    ((IDbDataAdapter)adapter).Fill(itemsDataSet);
                    DataTable products = itemsDataSet.Tables["Items"];
                    products.Rows.Add(new object[] { 4, "New Row Added", 100, 100, 100 });
                    products.Rows[3].Delete();
                    DbCommand insertCommandWrapper = db.GetStoredProcCommandWithSourceColumns("AddItem", "ItemID", "ItemDescription", "Price", "QtyinHand", "QtyRequired");
                    DbCommand deleteCommandWrapper = db.GetStoredProcCommandWithSourceColumns("deleteItem", "ItemID");
                    DbCommand updateCommandWrapper = db.GetStoredProcCommandWithSourceColumns("UpdateItem", "ItemID", "ItemDescription");
                    insertCommandWrapper.Connection = conn;
                    insertCommandWrapper.Transaction = trans;
                    deleteCommandWrapper.Connection = conn;
                    deleteCommandWrapper.Transaction = trans;
                    updateCommandWrapper.Connection = conn;
                    updateCommandWrapper.Transaction = trans;
                    ((IDbDataAdapter)adapter).InsertCommand = insertCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).DeleteCommand = deleteCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).UpdateCommand = updateCommandWrapper;//.Command;
                    adapter.Update(itemsDataSet);
                    trans.Commit();
                }
            }
            DataSet da = new DataSet();
            da.ReadXml(itemsXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            Assert.AreEqual(da.Tables[0].Rows.Count, itemsDataSet.Tables[0].Rows.Count);
            for (index = 0; index < da.Tables[0].Rows.Count; index++)
            {
                Assert.AreEqual(da.Tables[0].Rows[index][0].ToString().Trim(), itemsDataSet.Tables[0].Rows[index][1].ToString().Trim());
            }
        }

        //(" DbDataAdapter.GetDataAdapter(UpdateBehavior) has protected acess")]
        [TestMethod]
        public void GetDataAdapterStandardTest()
        {
            Database db = DatabaseFactory.CreateDatabase("DataSQLTest");
            DataSet itemsDataSet = new DataSet();
            DbCommand command = db.GetStoredProcCommand("ItemsGet");
            //DbDataAdapter adapter = db.GetDataAdapter(UpdateBehavior.Standard)
            using (DbDataAdapter adapter = db.GetDataAdapter())//UpdateBehavior.Standard))
            {
                using (DbConnection conn = db.CreateConnection())
                {
                    conn.Open();
                    DbTransaction trans = conn.BeginTransaction();
                    command.Connection = conn;
                    ((IDbDataAdapter)adapter).SelectCommand = command;//.Command;
                    command.Transaction = trans;
                    adapter.TableMappings.Add("Table", "Items");
                    ((IDbDataAdapter)adapter).Fill(itemsDataSet);
                    DataTable products = itemsDataSet.Tables["Items"];
                    products.Rows.Add(new object[] { 4, "New Row Added", 100, 100, 100 });
                    products.Rows[3].Delete();
                    DbCommand insertCommandWrapper = db.GetStoredProcCommandWithSourceColumns("AddItem", "ItemID", "ItemDescription", "Price", "QtyinHand", "QtyRequired");
                    DbCommand deleteCommandWrapper = db.GetStoredProcCommandWithSourceColumns("deleteItem", "ItemID");
                    DbCommand updateCommandWrapper = db.GetStoredProcCommandWithSourceColumns("UpdateItem", "ItemID", "ItemDescription");
                    insertCommandWrapper.Connection = conn;
                    insertCommandWrapper.Transaction = trans;
                    deleteCommandWrapper.Connection = conn;
                    deleteCommandWrapper.Transaction = trans;
                    updateCommandWrapper.Connection = conn;
                    updateCommandWrapper.Transaction = trans;
                    ((IDbDataAdapter)adapter).InsertCommand = insertCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).DeleteCommand = deleteCommandWrapper;//.Command;
                    ((IDbDataAdapter)adapter).UpdateCommand = updateCommandWrapper;//.Command;
                    adapter.Update(itemsDataSet);
                    trans.Commit();
                }
            }
            DataSet da = new DataSet();
            da.ReadXml(itemsXMLfile);
            int count = da.Tables[0].Rows.Count;
            int index = 0;
            Assert.AreEqual(da.Tables[0].Rows.Count, itemsDataSet.Tables[0].Rows.Count);
            for (index = 0; index < da.Tables[0].Rows.Count; index++)
            {
                Assert.AreEqual(da.Tables[0].Rows[index][0].ToString().Trim(), itemsDataSet.Tables[0].Rows[index][1].ToString().Trim());
            }
        }
    }
}
