using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for CustomHandlerDicTestFixture
    /// </summary>
    [TestClass]
    public class CustomHandlerDicTestFixture
    {
        private const string fileName = "CustomHandler.txt";
        private bool fileLogged;
        private DictionaryConfigurationSource configSource;
        private ExceptionPolicyFactory factory;
        public CustomHandlerDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        [TestInitialize()]
        public void MyTestInitialize() {
            if (File.Exists(fileName))
                File.Delete(fileName);

            DictionarySourceSection sourceSection = new DictionarySourceSection();
            configSource = sourceSection.BuildDictionarySourceSection();
            factory = new ExceptionPolicyFactory(configSource);    
        }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void CustomThrowNewDicTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: DBConcurrency Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Data.DBConcurrencyException(exceptionMessage);
                ExceptionPolicyImpl epi = factory.Create("Custom Handler Dic");
                rethrow = epi.HandleException(originalException); 
                
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual(exceptionMessage, e.Message);
                Assert.AreEqual("DBConcurrencyException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void WrapCustomThrowNewDicTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: File Not Found Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.IO.FileNotFoundException(exceptionMessage);
                ExceptionPolicyImpl epi = factory.Create("Wrap Custom Handler Dic");
                rethrow = epi.HandleException(originalException); 

                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("File Not Found Exception wrapped with Security Exception", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
                Assert.AreEqual(exceptionMessage, e.InnerException.Message);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("File Not Found Exception wrapped with Security Exception") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void CustomWrapThrowNewDicTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: File Not Found Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.IO.FileNotFoundException(exceptionMessage);
                ExceptionPolicyImpl epi = factory.Create("Custom Wrap Handler Dic");
                rethrow = epi.HandleException(originalException); 
                
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("File Not Found Exception wrapped with Security Exception", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
                Assert.AreEqual("Original Exception: File Not Found Exception", e.InnerException.Message);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void ReplaceCustomThrowNewDicTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;
            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                ExceptionPolicyImpl epi = factory.Create("Replace Custom Handler Dic");
                rethrow = epi.HandleException(originalException); 

                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains("Replaced Exception: Replace Security with Verification") == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }

        [TestMethod]
        public void CustomReplaceThrowNewDicTest()
        {
            bool exceptionRaised = false;
            string exceptionMessage = "Original Exception: Security Exception";
            bool rethrow = false;

            try
            {
                Exception originalException = new System.Security.SecurityException(exceptionMessage);
                ExceptionPolicyImpl epi = factory.Create("Custom Replace Handler Dic");
                rethrow = epi.HandleException(originalException); 
                
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;

                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }

            if (File.Exists(fileName))
            {
                fileLogged = true;
                if (File.ReadAllText(fileName).Contains(exceptionMessage) == false)
                    Assert.Fail("Exception not handled by custom handler");
            }
            else
                fileLogged = false;

            Assert.AreEqual(true, exceptionRaised);
            Assert.AreEqual(true, fileLogged);
        }
    }
}
