using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for ExceptionHandlingDictSourceTestFixture
    /// </summary>
    [TestClass]
    public class ExceptionHandlingDictSourceTestFixture
    {
        private DictionaryConfigurationSource configSource;
        private ExceptionPolicyFactory factory;
        public ExceptionHandlingDictSourceTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            DictionarySourceSection sourceSection = new DictionarySourceSection();
            configSource = sourceSection.BuildDictionarySourceSection();
            factory = new ExceptionPolicyFactory(configSource);    
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        [TestMethod]
        public void NoExceptionTypeDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Test No ExceptionType in Config File");

            ExceptionPolicyImpl epi = factory.Create("No Exception Type Dic");
            bool rethrow = epi.HandleException(originalException);
            Assert.IsTrue(rethrow);

        }

        [TestMethod]
        public void NoHandlerwithRethrowDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Test No Handler with Posthandling Rethrow in Config");


            ExceptionPolicyImpl epi = factory.Create("No Handler with Rethrow Dic");
            bool rethrow = epi.HandleException(originalException);
            Assert.IsTrue(rethrow);


        }

        [TestMethod]
        [ExpectedException(typeof(System.Security.SecurityException))]
        public void NoHandlerwithThrowNewDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("No handlers with Posthandling ThrowNewException in Config");

            ExceptionPolicyImpl epi = factory.Create("No Handler with Throw New Dic");
            bool rethrow = epi.HandleException(originalException);
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void NoHandlerwithNoneDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");

            ExceptionPolicyImpl epi = factory.Create("No Handler with None Dic");
            bool rethrow = epi.HandleException(originalException);
            Assert.IsFalse(rethrow);
        }

        [TestMethod]
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void NoPolicyDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Policy not present");

            ExceptionPolicyImpl epi = factory.Create("Policy does not exist");
            bool rethrow = epi.HandleException(originalException); 
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void InvalidHandlerDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("No Type Specified in Handler");

            ExceptionPolicyImpl epi = factory.Create("Invalid Handler Dic");
            bool rethrow = epi.HandleException(originalException);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void WrapWithNoExceptionTypeTest()
        {
            Exception originalException = new System.Security.SecurityException("No Exception Type Specified in Handler");

            ExceptionPolicyImpl epi = factory.Create("Wrap Handler without Type Dic");
            bool rethrow = epi.HandleException(originalException);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void ReplaceWithNoExceptionTypeTest()
        {
            Exception originalException = new System.Security.SecurityException("No Exception Type Specified in Handler");

            ExceptionPolicyImpl epi = factory.Create("Replace Handler without Type Dic");
            bool rethrow = epi.HandleException(originalException);
        }

        

    }
}
