using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Diagnostics;
using System.Configuration;
using System.Messaging;
using System.Management;
using System.Threading;
using System.Collections.Specialized;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;



namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for ExceptionInstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class ExceptionInstrumentationTestFixture
    {
        String wmiText = "";
        bool wmiLogged = false;
        public ExceptionInstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void ExceptionHandledCounterTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            string counterName = "Exceptions Handled/sec";
            int initialCount = GetCounterValue(counterName);

            try
            {
                Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by Data Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Wrap ThrowNew Same Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception 1: A recoverable error occurred while attempting to access the database.", e.Message);
                Assert.AreEqual("Original Exception: Concurrency Exception wrapped by Data Exception", e.InnerException.Message);
                Assert.AreEqual("DataException", e.GetType().Name);
                Assert.AreEqual("DBConcurrencyException", e.InnerException.GetType().Name);


            }

            Assert.AreEqual(true, exceptionRaised);

            int loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);

        }



        [TestMethod]
        public void HandlersExecutedCounterTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification which is again replaced with Host Protection Exception");
            bool rethrow = false;
            string counterName = "Exception Handlers Executed/sec";
            int initialCount = GetCounterValue(counterName);
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Replace Replace Handler");
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Replace Verification with HostProtection", e.Message);
                Assert.AreEqual("HostProtectionException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

            int loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 2, initialCount);
        }
        private int iWmiEventCount = 0;
        private static object lockObject = new object();
        [TestMethod]
        public void HandlingExceptionWMITest()
        {
            bool rethrow = false;
            string WMIEvent = "ExceptionHandlingFailureEvent";
            const string WmiPath = @"\root\EnterpriseLibrary";
            ManagementEventWatcher watcher = new ManagementEventWatcher();
            ManagementScope scope = new ManagementScope(@"\\." + WmiPath);
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM " + WMIEvent);
            watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrivedConfig);
            watcher.Start();
            for (int i = 0; i < 5; i++)
            {
                try
                {
                    Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred which is not handled");
                    rethrow = ExceptionPolicy.HandleException(originalException, "Wrap And Handling Exception Occurs In Chain Test");
                    if (rethrow) throw originalException;

                }
                catch (ExceptionHandlingException ex)
                {



                }
            }
            Thread.Sleep(1000);
            watcher.Dispose();
            Assert.AreEqual(5, iWmiEventCount);
        }

        [TestMethod]
        public void ConfigFailWMITest()
        {
            bool rethrow = false;
            wmiLogged = false;
            wmiText = "";

            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM ExceptionHandlingConfigurationFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();

            try
            {
                Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by Data Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Non Existent Handler");
                if (rethrow) throw originalException;
            }
            catch (ConfigurationErrorsException)
            {



            }

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("ExceptionHandlingConfigurationFailureEvent") == -1)
            {
                Assert.Fail();
            }
        }

        [TestMethod]
        public void ConfigFailEventLogTest()
        {
            bool rethrow = false;

            try
            {
                Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by Data Exception");
                rethrow = ExceptionPolicy.HandleException(originalException, "Non Existent Handler");
                if (rethrow) throw originalException;
            }
            catch (ConfigurationErrorsException)
            {
            }

            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("Non Existent Handler")))
                    Assert.Fail("Event Not logged");
            }

        }

        [TestMethod]
        public void FailureEventLogTest()
        {
            try
            {
                Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");
                bool rethrow = ExceptionPolicy.HandleException(originalException, "Exception Handling Failure Event Test");
            }
            catch (ExceptionHandlingException)
            { }
            string logName = EventLog.LogNameFromSourceName("Application", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;
                if (!(eventlog.Entries[eventlog.Entries.Count - 1].Message.Contains("Raise Exception Handler Failure")))
                    Assert.Fail("Event Not logged");
            }

        }

        [TestMethod]
        public void FailureEventWMITest()
        {
            wmiLogged = false;
            wmiText = "";

            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM ExceptionHandlingFailureEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();

            try
            {
                Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");
                bool rethrow = ExceptionPolicy.HandleException(originalException, "Exception Handling Failure Event Test");
            }
            catch (ExceptionHandlingException)
            { }
            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("Raise Exception Handler Failure") == -1)
            {
                Assert.Fail();
            }

        }
        private void WatcherEventArrivedConfig(object sender, EventArrivedEventArgs args)
        {
            lock (lockObject)
            {
                iWmiEventCount++;
            }

        }

        private int GetCounterValue(string counterName)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Exception Handling Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("Total");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            /// Validate the message here
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }


    }

    [ConfigurationElementType(typeof(CustomHandlerData))]
    public class CustomExceptionHandlerThrower : IExceptionHandler
    {

        public static bool exceptionOccurred = false;
        private const string fileName = "CustomHandler.txt";

        public CustomExceptionHandlerThrower()
        {
        }

        public CustomExceptionHandlerThrower(NameValueCollection attributes)
            : this()
        {
        }

        public Exception HandleException(Exception exception, Guid correlationID)
        {
            throw new ApplicationException("Raise Exception Handler Failure");
        }
    }
}
