using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for ReplaceHandlerTestFixture
    /// </summary>
    [TestClass]
    public class ReplaceHandlerDictTestFixture
    {
        private DictionaryConfigurationSource configSource;
        string connString;
        private ExceptionPolicyFactory factory;
        public ReplaceHandlerDictTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            //  TODO: Add test initialization code
            DictionarySourceSection sourceSection = new DictionarySourceSection();
            configSource = sourceSection.BuildDictionarySourceSection();
            factory = new ExceptionPolicyFactory(configSource);    

       }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        #region "Replace Test Cases"
        [TestMethod]
        public void ReplaceThrowNewSameAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");

            bool rethrow = false;
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Replace ThrowNew Same Assembly Dic");
                rethrow = epi.HandleException(originalException);
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Security Exception Replaced with Verification Exception", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }

        [TestMethod]
        public void ReplaceThrowNewDiffAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced by verification Exception");

            bool rethrow = false;

            try
            {
                ExceptionPolicyImpl epi = factory.Create("Replace ThrowNew Diff Assembly Dic");
                rethrow = epi.HandleException(originalException);
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception replaced by Application Exception", e.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }

        [TestMethod]
        public void ReplaceRethrowDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception rethrown");
            bool rethrow = false;

            ExceptionPolicyImpl epi = factory.Create("Replace Rethrow Dic");
            rethrow = epi.HandleException(originalException);
            Assert.IsTrue(rethrow);

        }

        [TestMethod]
        public void ReplaceExceptionWithNoneHandlerDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");
            bool rethrow = false;

            ExceptionPolicyImpl epi = factory.Create("Replace with None Dic");
            rethrow = epi.HandleException(originalException);
            Assert.IsFalse(rethrow);

        }

        [TestMethod]
        public void ReplaceUnexpectedExceptionDictSourceTest()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");

            bool rethrow = false;

            ExceptionPolicyImpl epi = factory.Create("Replace But Exception Type Not in Policy Dict");
            rethrow = epi.HandleException(originalException);
            Assert.IsTrue(rethrow);

        }

        [TestMethod]
        public void ReplaceMultipleExceptionHandlerDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Data.StrongTypingException("Original Exception: Strong Typing Exception replaced with Not Implemented Exception");

            bool rethrow = false;
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Replace Multiple Exception Handler Dic");
                rethrow = epi.HandleException(originalException);

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Exception replaced with Not Implemented Exception", e.Message);
                Assert.AreEqual("NotImplementedException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);


            exceptionRaised = false;
            rethrow = false;
            originalException = new System.IO.FileNotFoundException("Original Exception: File Not Found replaced with Application Exception");
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Replace Multiple Exception Handler Dic");
                rethrow = epi.HandleException(originalException);
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("File Not Found replaced with Application Exception", e.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }

        #endregion

        #region "Mutiple Handlers"
        [TestMethod]
        public void ReplaceReplaceHandlerDictSourceTest()
        {
            bool exceptionRaised = false;

            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification which is again replaced with Host Protection Exception");

            bool rethrow = false;
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Replace Replace Handler Dic");
                rethrow = epi.HandleException(originalException);
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Replace Verification with HostProtection", e.Message);
                Assert.AreEqual("HostProtectionException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }


        [TestMethod]
        public void ReplaceWrapHandlerDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification exception which is wrapped with XML Syntax exception");

            bool rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Replace Wrap Handler Dic");
                rethrow = epi.HandleException(originalException);
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception: Wrap Verification with Xml Syntax", e.Message);
                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.InnerException.Message);
                Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                Assert.AreEqual("VerificationException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }
        #endregion
    }
}
