using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;
using System.Configuration;
using System.Messaging;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for ReplaceHandlerTestFixture
    /// </summary>
    [TestClass]
    public class ReplaceHandlerTestFixture
    {
    
        public ReplaceHandlerTestFixture()
        {
 
        }

        [TestInitialize()]
        public void Initialize()
        {
           

        }

 
        [TestCleanup()]
        public void Cleanup()
        {
        }


        #region "Replace Test Cases"
        [TestMethod]
        public void ReplaceThrowNewSameAssemblyTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            try
            {
               Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");
               rethrow = ExceptionPolicy.HandleException(originalException, "Replace ThrowNew Same Assembly");
               if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Security Exception Replaced with Verification Exception", e.Message);
                Assert.AreEqual("VerificationException", e.GetType().Name);
            }
                              
            Assert.AreEqual(true, exceptionRaised);
        }


        [TestMethod]
        public void ReplaceThrowNewDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced by verification Exception");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Replace ThrowNew Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception replaced by Application Exception", e.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }


        [TestMethod]
        public void ReplaceRethrowTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception rethrown");
            
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Replace Rethrow");
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void ReplaceExceptionWithNoneHandlerTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Replace with None");
            Assert.IsFalse(rethrow);
        }
        
        [TestMethod]
        public void ReplaceUnexpectedException()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Replace But Exception Type Not in Policy");
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void ReplaceMultipleExceptionHandlersTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Data.StrongTypingException("Original Exception: Strong Typing Exception replaced with Not Implemented Exception");
            
            bool rethrow = false;
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Replace Multiple Exception Handler");
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Exception replaced with Not Implemented Exception", e.Message);
                Assert.AreEqual("NotImplementedException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

            exceptionRaised = false;
            rethrow = false;

            originalException = new System.IO.FileNotFoundException("Original Exception: File Not Found replaced with Application Exception");
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Replace Multiple Exception Handler");
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("File Not Found replaced with Application Exception", e.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);



        }

        [TestMethod]
        public void ReplaceExceptionCreateDirect()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");
            ReplaceHandler handler = new ReplaceHandler("Replaced Exception: Arithmetic Exception replaced with Security Exception", Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
            Exception replacedException = handler.HandleException(originalException, new Guid());    
            Assert.IsTrue(replacedException is System.Security.SecurityException);
            Assert.AreEqual(replacedException.Message,"Replaced Exception: Arithmetic Exception replaced with Security Exception");
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]   
        public void ReplaceHandlerExceptionTest()
        {
            Exception originalException = new System.ArithmeticException("Arithmetic Exception is not handled");
            ReplaceHandler handler = new ReplaceHandler("Replaced Exception: Arithmetic Exception replaced with Security Exception", Type.GetType("System.String, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"));
        }

        #endregion

        #region "Mutiple Handlers"
        [TestMethod]
        public void ReplaceReplaceHandlerTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification which is again replaced with Host Protection Exception");
            bool rethrow = false;
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Replace Replace Handler");
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Replaced Exception: Replace Verification with HostProtection", e.Message);
                Assert.AreEqual("HostProtectionException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }


        [TestMethod]
        public void ReplaceWrapHandlerTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification exception which is wrapped with XML Syntax exception");
            bool rethrow = false;
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Replace Wrap Handler");
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception: Wrap Verification with Xml Syntax", e.Message);
                Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.InnerException.Message);
                Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                Assert.AreEqual("VerificationException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
        }
        #endregion


    }
}
