using System;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;
using System.Collections.Generic;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;

namespace ExceptionHandlingCoreTests
{
    /// <summary>
    /// Summary description for WrapHandlerDictSourceTestFixture
    /// </summary>
    [TestClass]
    public class WrapHandlerDictSourceTestFixture
    {
        private DictionaryConfigurationSource configSource;
        private ExceptionPolicyFactory factory;
     
        public WrapHandlerDictSourceTestFixture()
        {
  
        }

        [TestInitialize()]
        public void Initialize()
        {
            DictionarySourceSection sourceSection = new DictionarySourceSection();
            configSource = sourceSection.BuildDictionarySourceSection();
            factory = new ExceptionPolicyFactory(configSource);    

            
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Wrap Test Cases"
        [TestMethod]
        public void WrapThrowNewSameAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by Data Exception");
            bool rethrow = false;
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Wrap ThrowNew Same Assembly Dic");
                rethrow = epi.HandleException(originalException);
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception 1: A recoverable error occurred while attempting to access the database.", e.Message);
                Assert.AreEqual("Original Exception: Concurrency Exception wrapped by Data Exception", e.InnerException.Message);
                Assert.AreEqual("DataException", e.GetType().Name);
                Assert.AreEqual("DBConcurrencyException", e.InnerException.GetType().Name);

            }
	        Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);



        }


        [TestMethod]
        public void WrapThrowNewDiffAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Data.DBConcurrencyException("Original Exception: Concurrency Exception wrapped by IO Exception");
            bool rethrow = false;
            try
            {
                ExceptionPolicyImpl epi = factory.Create("Wrap ThrowNew Different Assembly Dic");
                rethrow = epi.HandleException(originalException);
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Wrapped Exception 2: A recoverable error occurred while attempting to access the database.", e.Message);
                Assert.AreEqual("Original Exception: Concurrency Exception wrapped by IO Exception", e.InnerException.Message);
                Assert.AreEqual("FileNotFoundException", e.GetType().Name);
                Assert.AreEqual("DBConcurrencyException", e.InnerException.GetType().Name);
            }
	    Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }
        
        [TestMethod]
        public void WrapRethrowDictSourceTest()
        {
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: Thread aborted exception");

            ExceptionPolicyImpl epi = factory.Create("Wrap Rethrow Dic");
            bool rethrow = epi.HandleException(originalException); 

            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void WrapwithNoneHandlerDictSourceTest()
        {
            Exception originalException = new System.Data.ReadOnlyException("Original Exception: Read Only Exception wrapped by Security Exception");

            ExceptionPolicyImpl epi = factory.Create("Wrap with None Dic");
            bool rethrow = epi.HandleException(originalException); 
            Assert.IsFalse(rethrow);

        }


        [TestMethod]
        public void WrapUnexpectedExceptionDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred which is not handled");

            ExceptionPolicyImpl epi = factory.Create("Wrap ThrowNew Same Assembly Dic");
            bool rethrow = epi.HandleException(originalException); 
            Assert.IsTrue(rethrow);
        }

        [TestMethod]
        public void WrapMultipleExceptionHandlerDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Data.StrongTypingException("Original Exception: Strong Typing Exception wrapped with Not Implemented Exception");
            bool rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Wrap Multiple Exception Handler Dic");
                rethrow = epi.HandleException(originalException); 
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Exception wrapped with Not Implemented Exception", e.Message);
                Assert.AreEqual("Original Exception: Strong Typing Exception wrapped with Not Implemented Exception", e.InnerException.Message);
                Assert.AreEqual("NotImplementedException", e.GetType().Name);
                Assert.AreEqual("StrongTypingException", e.InnerException.GetType().Name);
            }

            Assert.IsFalse(rethrow);

            Assert.AreEqual(true, exceptionRaised);

            exceptionRaised = false;
            originalException = new System.IO.FileNotFoundException("Original Exception: File Not Found wrapped with Application Exception");
            rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Wrap Multiple Exception Handler Dic");
                rethrow = epi.HandleException(originalException); 
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("File Not Found wrapped with Application Exception", e.Message);
                Assert.AreEqual("Original Exception: File Not Found wrapped with Application Exception", e.InnerException.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }

        #endregion

        #region "Multiple Handlers"
        [TestMethod]
        public void WrapWrapThrowNewSameAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by Verification Exception which is wrapped by XMLSyntaxException");
            bool rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Wrap Wrap ThrowNew Same Assembly Dic");
                rethrow = epi.HandleException(originalException); 
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Wrapping level 2: Verification Exception Wrapped with XML Syntax Exception", e.Message);
                Assert.AreEqual("Wrapping level 1: Security Exception Wrapped with Verification Exception", e.InnerException.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by Verification Exception which is wrapped by XMLSyntaxException", e.InnerException.InnerException.Message);
                Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                Assert.AreEqual("VerificationException", e.InnerException.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void WrapWrapThrowNewDiffAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is wrapped by Application Exception");
            bool rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Wrap Wrap ThrowNew Diff Assembly Dic");
                rethrow = epi.HandleException(originalException); 
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("IO Exception wrapped with Application Exception", e.Message);
                Assert.AreEqual("Security Exception Wrapped with IO Exception", e.InnerException.Message);
                Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is wrapped by Application Exception", e.InnerException.InnerException.Message);
                Assert.AreEqual("ApplicationException", e.GetType().Name);
                Assert.AreEqual("FileNotFoundException", e.InnerException.GetType().Name);
                Assert.AreEqual("SecurityException", e.InnerException.InnerException.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }


        [TestMethod]
        public void WrapWrapRethrowDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception");

            ExceptionPolicyImpl epi = factory.Create("Wrap Wrap Rethrow Diff Assembly Dic");
            bool rethrow = epi.HandleException(originalException); 

	    Assert.IsTrue(rethrow);
        }


        [TestMethod]
        public void WrapReplaceHandlerDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: FileNotFound Exception wrapped by Security Exception which is replaced by Data Exception");
            
            bool rethrow = false;
            try
            {

                ExceptionPolicyImpl epi = factory.Create("Wrap Replace Handler Dic");
                rethrow = epi.HandleException(originalException);
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Security Exception wrapped with DataException", e.Message);
                Assert.AreEqual("DataException", e.GetType().Name);
            }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

        }


        
        #endregion
    }
}
