using System;
using System.Collections.Specialized;
using System.Globalization;
using System.IO;
using System.Reflection;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Properties;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;

namespace ExceptionHandlingLoggingTests
{
    public class CustomExceptionFormatter : ExceptionFormatter
    {
        private readonly TextWriter writer;
        
        public CustomExceptionFormatter(TextWriter writer, Exception exception)
            : base(exception)
        {
			if (writer == null) throw new ArgumentNullException("writer");

            this.writer = writer;
        }

        public TextWriter Writer
        {
            get { return this.writer; }
        }

        protected override void WriteDescription()
        {
             
            string separator = new string('*', 80);

            this.Writer.WriteLine(separator); 
            
            this.Writer.Write("Exception Description : ");
            this.Writer.WriteLine(base.Exception.GetType().FullName);

            this.Writer.WriteLine(separator);
        }

        protected override void WriteDateTime(DateTime utcNow)
        {

        }

        protected override void WriteExceptionType(Type exceptionType)
        {
            string separator = new string('*', 80);
            
            this.Writer.Write("Exception type : ");
            this.Writer.WriteLine(exceptionType.AssemblyQualifiedName);

            this.Writer.WriteLine(separator);
        }

        protected override void WriteMessage(string message)
        {
            string separator = new string('*', 80);

            this.Writer.Write("Message : ");
            this.Writer.WriteLine(message);

            this.Writer.WriteLine(separator);
        }

        protected override void WriteSource(string source)
        {
        }

        protected override void WriteHelpLink(string helpLink)
        {
        }

        protected override void WritePropertyInfo(PropertyInfo propertyInfo, object value)
        {

        }

        protected override void WriteFieldInfo(FieldInfo fieldInfo, object value)
        {

        }

        protected override void WriteStackTrace(string stackTrace)
        {

        }

        protected override void WriteAdditionalInfo(NameValueCollection additionalInformation)
        {
            
        }

       
    }
}
