using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Filters;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation.Configuration;

namespace ExceptionHandlingLoggingTests
{
    public class DictionarySourceLogging
    {
        public DictionaryConfigurationSource BuildDictionarySourceSection()
        {
            DictionaryConfigurationSource section = new DictionaryConfigurationSource();
            ExceptionHandlingSettings setting = new ExceptionHandlingSettings();

           
            #region "Replace Handler Policies"
            ExceptionPolicyData epd = new ExceptionPolicyData("Replace Log To File ThrowNew");
            ExceptionTypeData etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            ReplaceHandlerData rhd = new ReplaceHandlerData();
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 6 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);

            LoggingExceptionHandlerData led = new LoggingExceptionHandlerData();
            led.Name = "Logging Handler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 145;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            led.Priority = 0;


            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);

            setting.ExceptionPolicies.Add(epd);

           
#endregion

            #region "Log Handler Policies"
            epd = new ExceptionPolicyData("Logging To Flat File PostHandingAction None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "Exception";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            led = new LoggingExceptionHandlerData("Logging Handler", "FileTextCategory", 100, System.Diagnostics.TraceEventType.Error, "Enterprise Library Exception Handling", Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null"), 0);
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.Title = "Enterprise Library Exception Handling";
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Flat File PostHandlingAction Throw New Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityExeption";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileXmlCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.XmlExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging without Valid FormatterType Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityExeption";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("System.Text.ASCIIEncoding");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Flat File PostHandlingAction ReThrow Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "RemotingException";
            etd.Type = Type.GetType("System.Runtime.Remoting.RemotingException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileWrapCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Flat File Diff Settings Dic");
            etd = new ExceptionTypeData();
            etd.Name = "DBConcurrencyException";
            etd.Type = Type.GetType("System.Data.DBConcurrencyException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 100;
            led.Severity= System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Event Sink PostHandlingAction None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "Exception";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            led = new LoggingExceptionHandlerData();
            led.Name = "Logging Handler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Event Sink PostHandlingAction Throw New Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "VerificationException";
            etd.Type = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventXmlCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.XmlExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority= 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Event Sink PostHandlingAction ReThrow Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "IOException";
            etd.Type = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.NotifyRethrow;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventWrapCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Logging To Event Sink Diff Settings Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 100;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Event Sink Wrap With Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 117;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            WrapHandlerData whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security Exception 2 is logged and is wrapped by Verification Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Event Sink Wrap With Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 116;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "IO Exception 2 is logged and is wrapped by Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Flat File Wrap With Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 116;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security Exception 3 is logged and is wrapped by Verification Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Flat File Wrap With Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 116;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "IO Exception 3 is logged and is wrapped by Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Event Sink Replace With Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 122;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 4 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Event Sink Replace With Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 122;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "IO Exception 2 is logged and is replaced by Security Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Flat File Replace With Same Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 122;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 4 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Log To Flat File Replace With Diff Assembly Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 122;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "IO Exception 3 is logged and is replaced by Security Exception";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("WrapReplaceLog Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "FileNotFoundException";
            etd.Type = Type.GetType("System.IO.FileNotFoundException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "File Not Found Exception wrapped with Security Exception";
            whd.WrapExceptionType = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security Exception replaced with DataException";
            rhd.ReplaceExceptionType = Type.GetType("System.Data.DataException, System.Data, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 122;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("ReplaceWrapLog Handler Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Replaced Exception: Replace Security with Verification";
            rhd.ReplaceExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Wrapped Exception: Wrap Verification with Xml Syntax";
            whd.WrapExceptionType = Type.GetType("System.Security.XmlSyntaxException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace And Log To Flat File With PostHandlingAction  Throw New Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 6 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace And Log To Flat File With PostHandlingAction  None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 5 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace And Log To Event Sink With PostHandlingAction  None Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 6 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Replace And Log To Event Sink With PostHandlingAction  Throw New Exception Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            rhd.Name = "Replace Handler";
            rhd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.ReplaceHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            rhd.ExceptionMessage = "Security exception 4 replaced by IO exception";
            rhd.ReplaceExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(rhd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap And Log To File NoneAction Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security exception 2 wrapped by Verification exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap And Log To File ThrowNew Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security exception 1 wrapped by Verification exception";
            whd.WrapExceptionType = Type.GetType("System.Security.VerificationException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "FileTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap And Log To Event Sink NoneAction Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.None;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security exception 3 wrapped by IO exception";
            whd.WrapExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            epd = new ExceptionPolicyData("Wrap And Log To Event Sink ThrowNew Dic");
            etd = new ExceptionTypeData();
            etd.Name = "SecurityException";
            etd.Type = Type.GetType("System.Security.SecurityException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.PostHandlingAction = PostHandlingAction.ThrowNewException;
            whd = new WrapHandlerData();
            whd.Name = "Wrap Handler";
            whd.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.WrapHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling");
            whd.ExceptionMessage = "Security exception 4 wrapped by IO exception";
            whd.WrapExceptionType = Type.GetType("System.IO.IOException, mscorlib, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089");
            etd.ExceptionHandlers.Add(whd);
            led = new LoggingExceptionHandlerData();
            led.Name = "LoggingHandler";
            led.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging.LoggingExceptionHandler, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Logging");
            led.LogCategory = "EventTextCategory";
            led.EventId = 157;
            led.Severity = System.Diagnostics.TraceEventType.Error;
            led.Title = "Enterprise Library Exception Handling";
            led.FormatterType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.TextExceptionFormatter, Microsoft.Practices.EnterpriseLibrary.ExceptionHandling, Version=1.1.0.0, Culture=neutral, PublicKeyToken=null");
            led.Priority = 0;
            etd.ExceptionHandlers.Add(led);
            epd.ExceptionTypes.Add(etd);
            setting.ExceptionPolicies.Add(epd);

            #endregion


            section.Add("exceptionHandling", setting);
            

            #region DictionarySource

            LoggingSettings settings = new LoggingSettings();

            CategoryFilterData categoryFilter = new CategoryFilterData();
            categoryFilter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Filters.CategoryFilter, Microsoft.Practices.EnterpriseLibrary.Logging");
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("FileTextCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("FileXmlCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("FileWrapCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("EventTextCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("EventXmlCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("EventWrapCategory"));
            categoryFilter.CategoryFilters.Add(new CategoryFilterEntry("EmailCategory"));
            categoryFilter.CategoryFilterMode = CategoryFilterMode.DenyAllExceptAllowed;
            categoryFilter.Name = "DeniedCategory";
            settings.DefaultCategory = "FileTextCategory";

            PriorityFilterData priorityFilter = new PriorityFilterData("Priority", 0);
            priorityFilter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Filters.PriorityFilter, Microsoft.Practices.EnterpriseLibrary.Logging");

            TextFormatterData textFormatter = new TextFormatterData("TextFormat", "EntLibLog{newline}{tab}{message}{newline}{tab}{severity}{newline}{tab}{eventid}{newline}{tab}{timestamp}{newline}{tab}{title}{newline}");
            textFormatter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Formatters.TextFormatter, Microsoft.Practices.EnterpriseLibrary.Logging");

            TextFormatterData xmlFormatter = new TextFormatterData("XmlFormat", "<EntLibLog>{newline}{tab}<message>{message}</message>{newline}{tab}<title>{title}</title>{newline}</EntLibLog>");
            xmlFormatter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Formatters.TextFormatter, Microsoft.Practices.EnterpriseLibrary.Logging");

            TextFormatterData wrapFormatter = new TextFormatterData("WrapFormat", "BEGIN{newline}{message}{newline}END {newline}BEGIN{newline}{title}{newline}END");
            wrapFormatter.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Formatters.TextFormatter, Microsoft.Practices.EnterpriseLibrary.Logging");

            FlatFileTraceListenerData fileTextListener = new FlatFileTraceListenerData();
            fileTextListener.Name = "FileTextListener";
            fileTextListener.FileName = "FlatFileTestOutput\\EntLibLog.txt";
            fileTextListener.TraceOutputOptions = System.Diagnostics.TraceOptions.Callstack;
            fileTextListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FlatFileTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileTextListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FlatFileTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileTextListener.Header = "*************HEADER***************";
            fileTextListener.Footer = "*************FOOTER***************";
            fileTextListener.Formatter = "TextFormat";

            FlatFileTraceListenerData fileXmlListener = new FlatFileTraceListenerData();
            fileXmlListener.Name = "FileXmlListener";
            fileXmlListener.FileName = "XmlTestOutput\\EntLibLog.xml";
            fileXmlListener.TraceOutputOptions = System.Diagnostics.TraceOptions.Callstack;
            fileXmlListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FlatFileTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileXmlListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FlatFileTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileXmlListener.Header = "*************HEADER***************";
            fileXmlListener.Footer = "*************FOOTER***************";
            fileXmlListener.Formatter = "XmlFormat";

            FlatFileTraceListenerData fileWrapListener = new FlatFileTraceListenerData();
            fileWrapListener.Name = "FileWrapListener";
            fileWrapListener.FileName = "WrapTestOutput\\EntLibLog.txt";
            fileWrapListener.TraceOutputOptions = System.Diagnostics.TraceOptions.Callstack;
            fileWrapListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FlatFileTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileWrapListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FlatFileTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            fileWrapListener.Header = "*************HEADER***************";
            fileWrapListener.Footer = "*************FOOTER***************";
            fileWrapListener.Formatter = "WrapFormat";

            FormattedEventLogTraceListenerData eventTextListener = new FormattedEventLogTraceListenerData();
            eventTextListener.Name = "EventTextListener";
            eventTextListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FormattedEventLogTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventTextListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FormattedEventLogTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventTextListener.Formatter = "TextFormat";
            eventTextListener.Source = "EntLib Tests";

            FormattedEventLogTraceListenerData eventXmlListener = new FormattedEventLogTraceListenerData();
            eventXmlListener.Name = "EventXmlListener";
            eventXmlListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FormattedEventLogTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventXmlListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FormattedEventLogTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventXmlListener.Formatter = "XmlFormat";
            eventXmlListener.Source = "EntLib Tests";

            FormattedEventLogTraceListenerData eventWrapListener = new FormattedEventLogTraceListenerData();
            eventWrapListener.Name = "EventWrapListener";
            eventWrapListener.Type = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners.FormattedEventLogTraceListener, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventWrapListener.ListenerDataType = Type.GetType("Microsoft.Practices.EnterpriseLibrary.Logging.Configuration.FormattedEventLogTraceListenerData, Microsoft.Practices.EnterpriseLibrary.Logging");
            eventWrapListener.Formatter = "WrapFormat";
            eventWrapListener.Source = "EntLib Tests";

            TraceSourceData fileTextSource = new TraceSourceData();
            fileTextSource.Name = "FileTextCategory";
            fileTextSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceSourceData fileXmlSource = new TraceSourceData();
            fileXmlSource.Name = "FileXmlCategory";
            fileXmlSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceSourceData fileWrapSource = new TraceSourceData();
            fileWrapSource.Name = "FileWrapCategory";
            fileWrapSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceSourceData eventTextSource = new TraceSourceData();
            eventTextSource.Name = "EventTextCategory";
            eventTextSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceSourceData eventXmlSource = new TraceSourceData();
            eventXmlSource.Name = "EventXmlCategory";
            eventXmlSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceSourceData eventWrapSource = new TraceSourceData();
            eventWrapSource.Name = "EventWrapCategory";
            eventWrapSource.DefaultLevel = System.Diagnostics.SourceLevels.Information;

            TraceListenerReferenceData FileTextReference = new TraceListenerReferenceData();
            FileTextReference.Name = "FileTextListener";
            TraceListenerReferenceData FileXmlReference = new TraceListenerReferenceData();
            FileXmlReference.Name = "FileXmlListener";
            TraceListenerReferenceData FileWrapReference = new TraceListenerReferenceData();
            FileWrapReference.Name = "FileWrapListener";
            TraceListenerReferenceData EventTextReference = new TraceListenerReferenceData();
            EventTextReference.Name = "EventTextListener";
            TraceListenerReferenceData EventXmlReference = new TraceListenerReferenceData();
            EventXmlReference.Name = "EventXmlListener";
            TraceListenerReferenceData EventWrapReference = new TraceListenerReferenceData();
            EventWrapReference.Name = "EventWrapListener";

            fileTextSource.TraceListeners.Add(FileTextReference);
            fileXmlSource.TraceListeners.Add(FileXmlReference);
            fileWrapSource.TraceListeners.Add(FileWrapReference);
            eventTextSource.TraceListeners.Add(EventTextReference);
            eventXmlSource.TraceListeners.Add(EventXmlReference);
            eventWrapSource.TraceListeners.Add(EventWrapReference);

            settings.LogFilters.Add(categoryFilter);
            settings.LogFilters.Add(priorityFilter);

            settings.TraceSources.Add(fileTextSource);
            settings.TraceSources.Add(fileXmlSource);
            settings.TraceSources.Add(fileWrapSource);
            settings.TraceSources.Add(eventTextSource);
            settings.TraceSources.Add(eventXmlSource);
            settings.TraceSources.Add(eventWrapSource);

            settings.Formatters.Add(textFormatter);
            settings.Formatters.Add(xmlFormatter);
            settings.Formatters.Add(wrapFormatter);

            settings.TraceListeners.Add(fileTextListener);
            settings.TraceListeners.Add(fileXmlListener);
            settings.TraceListeners.Add(fileWrapListener);
            settings.TraceListeners.Add(eventTextListener);
            settings.TraceListeners.Add(eventXmlListener);
            settings.TraceListeners.Add(eventWrapListener);

            #endregion

            section.Add("loggingConfiguration", settings);

            InstrumentationConfigurationSection ics = new InstrumentationConfigurationSection(true, true, true);
            section.Add("instrumentationConfiguration", ics);
            return section;

        }
    }
}
