using System;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using System.Data.SqlClient;
using System.IO;
using System.Configuration;
using System.Diagnostics;
using System.Collections.Generic;
using System.Messaging;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;

namespace ExceptionHandlingLoggingTests
{
    /// <summary>
    /// Summary description for LoggingUsingDictSource
    /// </summary>
    [TestClass]
    public class LoggingDicTestFixture
    {
        private DictionaryConfigurationSource configSource;
        private ExceptionPolicyFactory factory;
        string msmqPath = null;
        public LoggingDicTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
            //  TODO: Add test initialization code

            DictionarySourceLogging sourceSection = new DictionarySourceLogging();
            configSource = sourceSection.BuildDictionarySourceSection();
            factory = new ExceptionPolicyFactory(configSource);    
            msmqPath = ".\\Private$\\myQueue";
            if (!MessageQueue.Exists(msmqPath))
                MessageQueue.Create(msmqPath);
        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }

        #region "Flat File"
        
        [TestMethod]
        public void LoggingFlatFileNoneActionDictionaryTest()
        {
            bool isPresent = false;
            Exception originalException= new System.Security.SecurityException("Security Exception logged in flat file");
            try
            {
              
                ExceptionPolicyImpl epi =
                    factory.Create("Logging To Flat File PostHandingAction None Dic");

                bool rethrow = epi.HandleException(originalException);

                Assert.IsFalse(rethrow);
            }
            catch (Exception e)
            {
                Console.Write(e.Message);
            }
                string filepath = AppDomain.CurrentDomain.BaseDirectory;
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }
        
        [TestMethod]
        public void LoggingFlatFileThrowNewDictionaryTest()
        {
            bool exceptionRaised = false;
            bool isPresent = false;
            bool rethrow = false;
            Exception originalException= new System.Security.SecurityException("Security Exception logged in flat file and thrown");
            try
                {
                    ExceptionPolicyImpl epi = factory.Create("Logging To Flat File PostHandlingAction Throw New Exception Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception logged in flat file and thrown", e.Message);
                    Assert.AreEqual("SecurityException", e.GetType().Name);
                }

            Assert.IsFalse(rethrow);
            
            if (File.Exists("XmlTestOutput\\EntLibLog.xml") == true)
            {
                isPresent = true;
                //File.Delete("XmlTestOutput\\EntLibLog.xml");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingFlatFileReThrowDictionaryTest()
        {
            bool isPresent = false;
            Exception originalException= new System.Runtime.Remoting.RemotingException("Remoting Exception logged in flat file and re-thrown");
            ExceptionPolicyImpl epi = factory.Create("Logging To Flat File PostHandlingAction ReThrow Exception Dic");
            bool rethrow = epi.HandleException(originalException);

            Assert.IsTrue(rethrow);            

            if (File.Exists("WrapTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("WrapTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);
 

        }

        [TestMethod]
        public void LoggingFlatFileDiffSettingsDictionaryTest()
        {
            bool isPresent = false;
            Exception originalException= new System.Data.DBConcurrencyException("DB Exception logged in flat file");
            ExceptionPolicyImpl epi = factory.Create("Logging To Flat File Diff Settings Dic");
            bool rethrow = epi.HandleException(originalException);

                Assert.IsFalse(rethrow);

            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        [ExpectedException(typeof(ExceptionHandlingException))]
        public void LoggingFlatFileInvalidFormatterDictionaryTest()
        {
           bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in flat file and thrown");

            ExceptionPolicyImpl epi = factory.Create("Logging without Valid FormatterType Dic");
            rethrow = epi.HandleException(originalException);
            if (rethrow) throw originalException;
        }
        #endregion

        #region EventSink

        [TestMethod]
        public void LoggingNoneActionEventSinkDictionaryTest()
        {
           Exception originalException= new System.Security.SecurityException("Security Exception logged in event sink");
           ExceptionPolicyImpl epi = factory.Create("Logging To Event Sink PostHandlingAction None Dic");
            bool rethrow = epi.HandleException(originalException);
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(100), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
        }

        [TestMethod]
        public void LoggingThrowNewEventSinkDictionaryTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.VerificationException("Verification Exception logged in event sink");
            try
                {
                    ExceptionPolicyImpl epi = factory.Create("Logging To Event Sink PostHandlingAction Throw New Exception Dic");
                     rethrow = epi.HandleException(originalException);

                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Verification Exception logged in event sink", e.Message);
                    Assert.AreEqual("VerificationException", e.GetType().Name);
                }
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(100), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingRethrowEventSinkDictionaryTest()
        {
            Exception originalException = new System.IO.IOException("IO Exception logged in event sink");

            
            ExceptionPolicyImpl epi = factory.Create("Logging To Event Sink PostHandlingAction ReThrow Exception Dic");
            bool rethrow = epi.HandleException(originalException);

            Assert.IsTrue(rethrow);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(100), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
    

        }

        [TestMethod]
        public void LoggingDiffSettingsEventSinkDictionaryTest()
        {
           Exception originalException = new System.IO.FileNotFoundException("File Not Found Exception logged in event sink");
           
            ExceptionPolicyImpl epi = factory.Create("Logging To Event Sink Diff Settings Dic");
            bool rethrow = epi.HandleException(originalException);

            Assert.IsFalse(rethrow);


            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual(eventlog.Entries[eventlog.Entries.Count - 1].Source, "EntLib Tests");
                Assert.AreEqual(eventlog.Entries[eventlog.Entries.Count - 1].InstanceId, Convert.ToInt64(100));


            }

        }


        #endregion

        #region LoggingAndWrapping

        [TestMethod]
        public void LoggingWrapEventSinkSameAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security exception 2 logged into event log");
            try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Event Sink Wrap With Same Assembly Dic");
                    rethrow = epi.HandleException(originalException);

                     if (rethrow) throw originalException;


                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 2 is logged and is wrapped by Verification Exception", e.Message);
                    Assert.AreEqual("Security exception 2 logged into event log", e.InnerException.Message);

                }

                
            
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(117), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);


            }

        }

        [TestMethod]
        public void LoggingWrapEventSinkDiffAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false; 
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 2 logged into event log");
              
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Event Sink Wrap With Diff Assembly Dic");
                    rethrow = epi.HandleException(originalException);

                    if (rethrow) throw originalException;


                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("IO Exception 2 is logged and is wrapped by Security Exception", e.Message);
                    Assert.AreEqual("File Not Found exception 2 logged into event log", e.InnerException.Message);

                }
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {
                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(116), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingWrapFileSinkSameAssemblyDictSourceTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false; 
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged into file");
            
                
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Flat File Wrap With Same Assembly Dic");
                    rethrow = epi.HandleException(originalException);
                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 3 is logged and is wrapped by Verification Exception", e.Message);
                    Assert.AreEqual("Security exception 3 logged into file", e.InnerException.Message);
                }
            Assert.IsFalse(rethrow);

            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void LoggingWrapFileSinkDiffAssemblyDictSourceTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false; 
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged into file");
            
                
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Flat File Wrap With Diff Assembly Dic");
                     rethrow = epi.HandleException(originalException);

                    if (rethrow) throw originalException;


                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("IO Exception 3 is logged and is wrapped by Security Exception", e.Message);
                    Assert.AreEqual("File Not Found exception 3 logged into file", e.InnerException.Message);

                }
                
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        #endregion

        #region LoggingAndReplacing

        [TestMethod]
        public void LoggingReplaceEventSinkSameAssemblyDictSourceTest()
        {
           bool rethrow = false; 
           bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 2 logged into event log");
            
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Event Sink Replace With Same Assembly Dic");
                   rethrow = epi.HandleException(originalException);

                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                }
           
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {
                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(122), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

        }

        [TestMethod]
        public void LoggingReplaceEventSinkDiffAssemblyDictSourceTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 2 logged into event log");
            
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Event Sink Replace With Diff Assembly Dic");
                    rethrow = epi.HandleException(originalException);
                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                }
            Assert.IsFalse(rethrow);

            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {
                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(122), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingReplaceFileSinkSameAssemblyDictSourceTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged into file");
               
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Flat File Replace With Same Assembly Dic");
                    rethrow = epi.HandleException(originalException);
                     if (rethrow) throw originalException;


                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                }
                
            
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void LoggingReplaceFileSinkDiffAssemblyDictSourceTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged into file");
             try
                {
                    ExceptionPolicyImpl epi = factory.Create("Log To Flat File Replace With Diff Assembly Dic");
                    rethrow = epi.HandleException(originalException);
                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);


        }


        #endregion

        #region WrapReplaceLog

        [TestMethod]
        public void WrapReplaceLogHandlerDictSourceTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: FileNotFound Exception wrapped by Security Exception which is replaced by Data Exception");
              
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("WrapReplaceLog Handler Dic");
                    rethrow = epi.HandleException(originalException);
                     if (rethrow) throw originalException;

                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                    Assert.AreEqual("IOException", e.GetType().Name);
                }
                
           
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void ReplaceWrapLogHandlerDictSourceTest()
        {

            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification exception which is wrapped with XML Syntax exception");
            
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("ReplaceWrapLog Handler Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Wrapped Exception: Wrap Verification with Xml Syntax", e.Message);
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.InnerException.Message);
                    Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                    Assert.AreEqual("IOException", e.InnerException.GetType().Name);
                }
                
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(157), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);


        }

        #endregion

        #region ReplaceAndLog

        [TestMethod]
        
        public void ReplaceLogToFileThrowNewDictSourceTest()
        {
            bool exceptionRaised = false;
            bool isPresent = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");
            
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Replace And Log To Flat File With PostHandlingAction  Throw New Exception Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                    Assert.AreEqual("IOException", e.GetType().Name);

                }
                
            Assert.IsFalse(rethrow);
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);

        }
        
        [TestMethod]
        public void ReplaceLogToFileNoneDictSourceTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");

            ExceptionPolicyImpl epi = factory.Create("Replace And Log To Flat File With PostHandlingAction  None Dic");
                bool rethrow = epi.HandleException(originalException);
                Assert.IsFalse(rethrow);
            
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        
        [TestMethod]
        public void ReplaceLogToEventSinkNoneActionDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced by IO Exception which is logged into event sink");
            
                bool rethrow = false;
                ExceptionPolicyImpl epi = factory.Create("Replace And Log To Event Sink With PostHandlingAction  None Dic");
                rethrow = epi.HandleException(originalException);

                Assert.IsFalse(rethrow);

                string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(157), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

        }

        
        [TestMethod]
        public void ReplaceLogToEventSinkThrowNewDictSourceTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced by IO Exception which is logged into event sink");
            
                bool rethrow = false;
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Replace And Log To Event Sink With PostHandlingAction  Throw New Exception Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                    Assert.AreEqual("IOException", e.GetType().Name);

                }
 	    Assert.IsFalse(rethrow);
        string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(157), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }

        #endregion

        #region WrapAndLog

        
        [TestMethod]
        public void WrapLogToFileThrowNewDictSourceTest()
        {
            bool exceptionRaised = false;
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");
            
                bool rethrow = false;
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Wrap And Log To File ThrowNew Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 1 wrapped by Verification exception", e.Message);
                    Assert.AreEqual("Original Exception: Security Exception Occurred", e.InnerException.Message);
                    Assert.AreEqual("VerificationException", e.GetType().Name);
                    Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
                }
                Assert.IsFalse(rethrow);
            
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);

        }
        
        [TestMethod]
        public void WrapLogToFileNoneDictSourceTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception Occurred");
            
                bool rethrow = false;
                ExceptionPolicyImpl epi = factory.Create("Wrap And Log To File NoneAction Dic");
                rethrow = epi.HandleException(originalException);

                Assert.IsFalse(rethrow);
            
            if (File.Exists("FlatFileTestOutput\\EntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\EntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

     
        [TestMethod]
        public void WrapLogToEventSinkNoneActionDictSourceTest()
        {
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink");

            ExceptionPolicyImpl epi = factory.Create("Wrap And Log To Event Sink NoneAction Dic");
                bool rethrow = epi.HandleException(originalException);

                Assert.IsFalse(rethrow);

                string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(157), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

        }

        [TestMethod]
        public void WrapLogToEventSinkThrowNewDictSourceTest()
        {
            bool exceptionRaised = false;
           Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink");
            
                bool rethrow = false;
                try
                {
                    ExceptionPolicyImpl epi = factory.Create("Wrap And Log To Event Sink ThrowNew Dic");
                    rethrow = epi.HandleException(originalException);
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 wrapped by IO exception", e.Message);
                    Assert.AreEqual("Original Exception: Security Exception wrapped by IO Exception which is logged into event sink", e.InnerException.Message);
                    Assert.AreEqual("IOException", e.GetType().Name);
                    Assert.AreEqual("SecurityException", e.InnerException.GetType().Name);
                }
            Assert.IsFalse(rethrow);

            string logName = EventLog.LogNameFromSourceName("EntLib Tests", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("EntLib Tests", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(157), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }
            Assert.AreEqual(true, exceptionRaised);

        }

        

        #endregion
    }
}
