using System;
using System.Management;
using System.Threading;
using System.Configuration;
using System.Data.SqlClient;
using System.IO;
using System.Diagnostics;
using System.Messaging;
using System.Runtime.Serialization.Formatters.Soap;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.EnterpriseLibrary.Logging;


namespace ExceptionHandlingLoggingTests
{
    /// <summary>
    /// Summary description for LoggingTestFixture
    /// </summary>
    [TestClass]
    public class LoggingTestFixture
    {
        string msmqPath;
        String wmiText = "";
        public LoggingTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        /// Initialize() is called once during test execution before
        /// test methods in this test class are executed.
        /// </summary>
        [TestInitialize()]
        public void Initialize()
        {
             msmqPath = ".\\Private$\\myqueue";
            if (!MessageQueue.Exists(msmqPath)) 
                MessageQueue.Create(msmqPath);

        }

        /// <summary>
        /// Cleanup() is called once during test execution after
        /// test methods in this class have executed unless the
        /// corresponding Initialize() call threw an exception.
        /// </summary>
        [TestCleanup()]
        public void Cleanup()
        {
            //  TODO: Add test cleanup code
        }
        [TestMethod]
        [ExpectedException(typeof(System.Configuration.ConfigurationErrorsException))]
        public void ExceptionHandlingExLoggingTest()
        {
            Exception originalException = new System.Security.SecurityException("No Handlers with Posthandling none in config");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Policy does not exist");


        }
  
#region "Flat File"
        [TestMethod]
        public void LoggingFlatFileNoneActionTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged to flat file");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandingAction None");
	        Assert.IsFalse(rethrow);
            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\SimpleEntLibLog.txt").Contains("Security Exception logged to flat file") == false)
                //    Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        
        [TestMethod]
        public void LoggingFlatFileThrowNewTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged in flat file and thrown");
            
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandlingAction Throw New Exception");

                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
            	exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in flat file and thrown", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }

            originalException = new System.Security.SecurityException("Security Exception logged in flat file again and thrown");
            
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandlingAction Throw New Exception");

                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in flat file again and thrown", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }

            originalException = new System.Security.SecurityException("Security Exception logged in flat file again and again and thrown");

            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandlingAction Throw New Exception");

                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged in flat file again and again and thrown", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }

            if (File.Exists("FlatFileTestOutput\\EntLibLog.xml") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\EntLibLog.xml").Contains("Security Exception logged in flat file and thrown") == false)
                //    Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\EntLibLog.xml");
            }
            Assert.AreEqual(true, isPresent);
            Assert.AreEqual(true, exceptionRaised);
            Assert.IsFalse(rethrow);
            
        }
        [TestMethod]
        public void LoggingFlatFileRethrowTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Runtime.Remoting.RemotingException("Remoting Exception logged in flat file and re-thrown");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandlingAction ReThrow Exception");
            Assert.IsTrue(rethrow);

            if (File.Exists("FlatFileTestOutput\\WrapEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\WrapEntLibLog.txt").Contains("Remoting Exception logged in flat file and re-thrown") == false)
                //    Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\WrapEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);


        }

        [TestMethod]
        public void LoggingFlatFileDiffSettingsTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Data.DBConcurrencyException("DB Exception logged in flat file");
            
	        bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File Diff Settings");
            Assert.IsFalse(rethrow);

            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\SimpleEntLibLog.txt").Contains("DB Exception logged in flat file") == false)
                //    Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void LoggingFlatFileNoneActionCustomFormatterTest()
        {
            bool isPresent = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged to flat file in Custom Format");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Flat File PostHandlingAction None in CustomFormat");
            Assert.IsFalse(rethrow);
            if (File.Exists("FlatFileTestOutput\\DictEntLibLog.txt") == true)
            {
                isPresent = true;
                //if (File.ReadAllText("FlatFileTestOutput\\DictEntLibLog.txt").Contains("Security Exception logged to flat file in Custom Format") == false)
                //    Assert.Fail("Exception not logged to flat file");
                //File.Delete("FlatFileTestOutput\\DictEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        #endregion

        #region "Event Sink"
        [TestMethod]
        public void LoggingNoneActionEventSinkTest()
        {
            Exception originalException = new System.Security.SecurityException("Security Exception logged in event sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Event Sink PostHandlingAction None");

            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(100), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);
                
            }


                
        }

        [TestMethod]
        public void LoggingThrowNewEventSinkTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.VerificationException("Verification Exception logged in event sink");
            try
            {
            	rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Event Sink PostHandlingAction Throw New Exception");
                if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Verification Exception logged in event sink", e.Message);
                    Assert.AreEqual("VerificationException", e.GetType().Name);
                }
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(215), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }

            Assert.AreEqual(true, exceptionRaised);

        }

        [TestMethod]
        public void LoggingRethrowEventSinkTest()
        {
            Exception originalException = new System.IO.IOException("IO Exception logged in event sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Event Sink PostHandlingAction ReThrow Exception");

            Assert.IsTrue(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");

            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(100),eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);

            }


        }
        

        [TestMethod]
        public void LoggingDiffSettingsEventSinkTest()
        {
            Exception originalException = new System.IO.FileNotFoundException("File Not Found Exception logged in event sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Event Sink Diff Settings");

            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging",".");

            using (EventLog eventlog = new EventLog(logName))
            {
                

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual(eventlog.Entries[eventlog.Entries.Count - 1].Source, "ExceptionHandlingLogging");
                Assert.AreEqual(eventlog.Entries[eventlog.Entries.Count - 1].InstanceId, Convert.ToInt64(100));


            }

        }


        
        #endregion
        
        #region "MSMQ Sink"

        [TestMethod]
        public void LoggingMSMQSinkNoneActionTest()
        {
            Exception originalException = new System.Security.SecurityException("Security Exception logged in MSMQ sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging to MSMQ Sink PostHandlingAction None");
            Assert.IsFalse(rethrow);

            MessageQueue msmq = new MessageQueue(msmqPath);
            msmq.MessageReadPropertyFilter.Priority = true;

            Message message = msmq.Peek(new TimeSpan(0, 0, 20));
            message.Formatter = new XmlMessageFormatter(new string[] { "System.String,mscorlib" });
            msmq.Purge();
            msmq.Close();
            msmq = null;

            if (!message.Body.ToString().Contains("HandlingInstanceID:"))
            {
                Assert.Fail("No message or Expected message not in queue");
            }
        }

        [TestMethod]
        public void LoggingMSMQSinkThrowNewTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found Exception logged in MSMQ sink");
            try
            {

                rethrow = ExceptionPolicy.HandleException(originalException, "Logging to MSMQ Sink PostHandlingAction Throw New Exception");
                if (rethrow) throw originalException;

            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("File Not Found Exception logged in MSMQ sink", e.Message);
                Assert.AreEqual("FileNotFoundException", e.GetType().Name);

            }
            Assert.IsFalse(rethrow);

            MessageQueue msmq = new MessageQueue(msmqPath);
            msmq.MessageReadPropertyFilter.Priority = true;

            Message message = msmq.Peek();
            message.Formatter = new XmlMessageFormatter(new string[] { "System.String,mscorlib" });
            msmq.Purge();
            msmq.Close();
            msmq = null;

            if (!message.Body.ToString().Contains("HandlingInstanceID:"))
            {
                Assert.Fail("No message or Expected message not in queue");
            }
            Assert.AreEqual(true, exceptionRaised);
        }

        [TestMethod]
        public void LoggingMSMQSinkRethrowTest()
        {
            Exception originalException = new System.Security.VerificationException("Verification Exception logged in MSMQ sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To MSMQ Sink PostHandlingAction ReThrow Exception");

            Assert.IsTrue(rethrow);

            MessageQueue msmq = new MessageQueue(msmqPath);
            msmq.MessageReadPropertyFilter.Priority = true;

            Message message = msmq.Peek();
            message.Formatter = new XmlMessageFormatter(new string[] { "System.String,mscorlib" });
            msmq.Purge();
            msmq.Close();
            msmq = null;

            if (!message.Body.ToString().Contains("HandlingInstanceID:"))
            {
                Assert.Fail("No message or Expected message not in queue");
                
            }
           
        }

        
        #endregion

        [TestMethod]
        public void LoggingNoneActionWMISinkTest()
        {
            wmiText = "";

            Exception originalException = new System.Security.SecurityException("Security Exception logged in WMI sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To WMI Sink PostHandlingAction None");
            Assert.IsFalse(rethrow);

                    }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            /// Validate the message here
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }
        #region "Email Sink"
        [TestMethod]
        public void LoggingEmailSinkNoneActionTest()
        {
            Exception originalException = new System.Security.SecurityException("Security Exception logged to Email Sink");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Email Sink PostHandlingAction None");
            Assert.IsFalse(rethrow);
        }
        [TestMethod]
        public void LoggingEmailSinkThrowNewTest()
        {
            bool exceptionRaised = false;
            bool rethrow = false;
            Exception originalException = new System.Security.SecurityException("Security Exception logged to email sink and thrown");
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Email Sink PostHandlingAction Throw New Exception");

                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception logged to email sink and thrown", e.Message);
                Assert.AreEqual("SecurityException", e.GetType().Name);
            }



            Assert.AreEqual(true, exceptionRaised);
            Assert.IsFalse(rethrow);
        }

        [TestMethod]
        public void LoggingEmailSinkRethrowTest()
        {
            Exception originalException = new System.Runtime.Remoting.RemotingException("Remoting Exception logged to email sink and re-thrown");
            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Email Sink PostHandlingAction ReThrow Exception");
            Assert.IsTrue(rethrow);


        }

        [TestMethod]
        public void LoggingEmailSinkDiffSettingsTest()
        {
            Exception originalException = new System.Data.DBConcurrencyException("DB Exception logged to Email Sink");

            bool rethrow = ExceptionPolicy.HandleException(originalException, "Logging To Email Sink Diff Settings");
            Assert.IsFalse(rethrow);

        }
        #endregion

        #region "Multiple Handlers"
        

        [TestMethod]
        public void LoggingWrapEventSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 2 logged into event log");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Event Sink Wrap With Same Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 2 is logged and is wrapped by Verification Exception", e.Message);
                    Assert.AreEqual("Security exception 2 logged into event log", e.InnerException.Message);

                }

                Assert.IsFalse(rethrow);

            
            Assert.AreEqual(true, exceptionRaised);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(117), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);


            }

        }

        [TestMethod]
        public void LoggingWrapEventSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 2 logged into event log");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Event Sink Wrap With Diff Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("IO Exception 2 is logged and is wrapped by Security Exception", e.Message);
                    Assert.AreEqual("File Not Found exception 2 logged into event log", e.InnerException.Message);

                }

                Assert.IsFalse(rethrow);

                string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(116), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);


            }
            Assert.AreEqual(true, exceptionRaised);
            


        }

        [TestMethod]
        public void LoggingWrapFileSinkSameAssemblyTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged into file");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Flat File Wrap With Same Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 3 is logged and is wrapped by Verification Exception", e.Message);
                    Assert.AreEqual("Security exception 3 logged into file", e.InnerException.Message);

                }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }
       
        [TestMethod]
        public void LoggingWrapFileSinkDiffAssemblyTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged into file");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Flat File Wrap With Diff Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("IO Exception 3 is logged and is wrapped by Security Exception", e.Message);
                    Assert.AreEqual("File Not Found exception 3 logged into file", e.InnerException.Message);

                }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);


        }

        [TestMethod]
        public void LoggingWrapEmailSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged to Email Sink");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Email Sink Wrap With Same Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception 3 is logged and is wrapped by Verification Exception", e.Message);
                Assert.AreEqual("Security exception 3 logged to Email Sink", e.InnerException.Message);

            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
           
        }

        [TestMethod]
        public void LoggingWrapEmailSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged to Email Sink");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Email Sink Wrap With Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("IO Exception 3 is logged and is wrapped by Security Exception", e.Message);
                Assert.AreEqual("File Not Found exception 3 logged to Email Sink", e.InnerException.Message);

            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            


        }

        [TestMethod]
        public void LoggingReplaceEmailSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged to email sink");

            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Email Sink Replace With Same Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("Security Exception 3 is logged and is replaced by Verification Exception", e.Message);
            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
           

        }

        [TestMethod]
        public void LoggingReplaceEmailSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged to email sink");
            bool rethrow = false;
            try
            {
                rethrow = ExceptionPolicy.HandleException(originalException, "Log To Email Sink Replace With Diff Assembly");
                if (rethrow) throw originalException;
            }
            catch (Exception e)
            {
                exceptionRaised = true;
                Assert.AreEqual("IO Exception 3 is logged and is replaced by Security Exception", e.Message);
            }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            
        }

        [TestMethod]
        public void LoggingReplaceEventSinkSameAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 2 logged into event log");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Event Sink Replace With Same Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 2 is logged and is replaced by Verification Exception", e.Message);
                }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(122), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);


            }

        }

        [TestMethod]
        public void LoggingReplaceEventSinkDiffAssemblyTest()
        {
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 2 logged into event log");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Event Sink Replace With Diff Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security exception 4 replaced by IO exception", e.Message);
                }

                  
            Assert.IsFalse(rethrow);
            string logName = EventLog.LogNameFromSourceName("ExceptionHandlingLogging", ".");
            using (EventLog eventlog = new EventLog(logName))
            {

                int entryCount = eventlog.Entries.Count;

                Assert.AreEqual("ExceptionHandlingLogging", eventlog.Entries[eventlog.Entries.Count - 1].Source);
                Assert.AreEqual(Convert.ToInt64(123), eventlog.Entries[eventlog.Entries.Count - 1].InstanceId);


            }
            Assert.AreEqual(true, exceptionRaised);



        }

        [TestMethod]
        public void LoggingReplaceFileSinkSameAssemblyTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Security exception 3 logged into file");
            
                bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Flat File Replace With Same Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception 3 is logged and is replaced by Verification Exception", e.Message);
                }

            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        [TestMethod]
        public void LoggingReplaceFileSinkDiffAssemblyTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("File Not Found exception 3 logged into file");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "Log To Flat File Replace With Diff Assembly");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("IO Exception 3 is logged and is replaced by Security Exception", e.Message);
                }

            Assert.IsFalse(rethrow);   
            Assert.AreEqual(true, exceptionRaised);
            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);


        }

        [TestMethod]
        public void WrapReplaceLogHandlerTest()
        {
            bool isPresent = false;
            bool exceptionRaised = false;
            Exception originalException = new System.IO.FileNotFoundException("Original Exception: FileNotFound Exception wrapped by Security Exception which is replaced by Data Exception");
                bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "WrapReplaceLog Handler");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Security Exception wrapped with DataException", e.Message);
                    Assert.AreEqual("DataException", e.GetType().Name);
                }
            Assert.IsFalse(rethrow);
            Assert.AreEqual(true, exceptionRaised);

            if (File.Exists("FlatFileTestOutput\\SimpleEntLibLog.txt") == true)
            {
                isPresent = true;
                //File.Delete("FlatFileTestOutput\\SimpleEntLibLog.txt");
            }
            Assert.AreEqual(true, isPresent);

        }

        

        [TestMethod]
        public void ReplaceWrapLogHandlerEmailTest()
        {

            bool exceptionRaised = false;
            Exception originalException = new System.Security.SecurityException("Original Exception: Security Exception replaced with Verification exception which is wrapped with XML Syntax exception");
            bool rethrow = false;
                try
                {
                    rethrow = ExceptionPolicy.HandleException(originalException, "ReplaceWrapLog Handler EmailSink");
                    if (rethrow) throw originalException;
                }
                catch (Exception e)
                {
                    exceptionRaised = true;
                    Assert.AreEqual("Wrapped Exception: Wrap Verification with Xml Syntax", e.Message);
                    Assert.AreEqual("Replaced Exception: Replace Security with Verification", e.InnerException.Message);
                    Assert.AreEqual("XmlSyntaxException", e.GetType().Name);
                    Assert.AreEqual("VerificationException", e.InnerException.GetType().Name);
                }
                Assert.IsFalse(rethrow);

            Assert.AreEqual(true, exceptionRaised);


        }
        
        #endregion
    }
}
