using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics; 
using System.Configuration; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;   
using CustomClasses;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.ObjectBuilder;  
namespace CustomClasses.Configuration
{
    [Assembler(typeof(ExtendedFlatFileTraceListenerAssembler))]
    public class ExtendedFlatFileTraceListenerData:FlatFileTraceListenerData 
    {
        private const string fileSize = "fileSize";
        public ExtendedFlatFileTraceListenerData()
            : base()
        {

        }

        [ConfigurationProperty(fileSize, IsRequired = false)]
        public int FileSize
        {
            get { return (int)base[fileSize]; }
            set { base[fileSize] = value; }
        }
    }

    public class ExtendedFlatFileTraceListenerAssembler : TraceListenerAsssembler
    {
        /// <summary>
        /// This method supports the Enterprise Library infrastructure and is not intended to be used directly from your code.
        /// Builds a <see cref="FlatFileTraceListener"/> based on an instance of <see cref="FlatFileTraceListenerData"/>.
        /// </summary>
        /// <seealso cref="TraceListenerCustomFactory"/>
        /// <param name="context">The <see cref="IBuilderContext"/> that represents the current building process.</param>
        /// <param name="objectConfiguration">The configuration object that describes the object to build. Must be an instance of <see cref="FlatFileTraceListenerData"/>.</param>
        /// <param name="configurationSource">The source for configuration objects.</param>
        /// <param name="reflectionCache">The cache to use retrieving reflection information.</param>
        /// <returns>A fully initialized instance of <see cref="FlatFileTraceListener"/>.</returns>
        public override TraceListener Assemble(IBuilderContext context, TraceListenerData objectConfiguration, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            ExtendedFlatFileTraceListenerData castedObjectConfiguration
                = (ExtendedFlatFileTraceListenerData)objectConfiguration;

            ILogFormatter formatter = GetFormatter(context, castedObjectConfiguration.Formatter, configurationSource, reflectionCache);

            TraceListener createdObject
                = new ExtendedFlatFileListener(
                    castedObjectConfiguration.FileName,
                    castedObjectConfiguration.Header,
                    castedObjectConfiguration.Footer,
                    formatter,castedObjectConfiguration.FileSize);

            return createdObject;
        }
    }

}
