using System;
using System.Text;
using System.Collections.Generic;
using System.Diagnostics;
using System.Messaging;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Logging.ExtraInformation;    
using Microsoft.Practices.EnterpriseLibrary.Logging.TraceListeners;
using Microsoft.Practices.EnterpriseLibrary.Logging.Formatters;
using Microsoft.Practices.EnterpriseLibrary.Logging.Filters; 
using Microsoft.Practices.EnterpriseLibrary.Logging.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;    
using CustomClasses;
namespace LoggerFacade
{
    /// <summary>
    /// Summary description for GeneralTestFixture
    /// </summary>
    [TestClass]
    public class GeneralTestFixture
    {
        public GeneralTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region CustomFactory Tests

        [TestMethod]
        public void CanCreateTraceListenerFromFactory()
        {
            TraceListenerCustomFactory factory = TraceListenerCustomFactory.Instance;
            TraceListener listener = (TraceListener)factory.CreateObject(null, "Event Log TraceListener", new SystemConfigurationSource(), null);
            LogEntry entry = new LogEntry();
            entry.Message = "Listener created using TraceListenerCustomFactory";
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                listener.TraceData(null, "Entlib Test", TraceEventType.Critical, 0, entry);
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                string loggedMessage = eventLog.Entries[finalCount - 1].Message;
                Assert.IsTrue(loggedMessage.Contains(entry.Message));   
            }

        }

        [TestMethod]
        public void CanCreateFormatterFromFactory()
        {
            LogFormatterCustomFactory factory = LogFormatterCustomFactory.Instance;
            ILogFormatter formatter = (ILogFormatter)factory.CreateObject(null, "Text Formatter", new SystemConfigurationSource(), null);
            LogEntry entry = new LogEntry();
            entry.Message = "The message to format the text";
            string result = formatter.Format(entry);
            TextFormatter formatter1 = new TextFormatter("Timestamp: {timestamp}\r\nMessage: {message}\r\nCategory: {category}\r\nPriority: {priority}\r\nEventId: {eventid}\r\nSeverity: {severity}\r\nTitle:{title}\r\nMachine: {machine}\r\nApp Domain: {appDomain}\r\nProcess Id: {processId}\r\nProcess Name: {processName}\r\nThread Name: {threadName}\r\nWin32 ThreadId: {win32ThreadId}\r\nExtended Properties: {dictionary({key} - {value}\r\n)}");
            Assert.IsTrue(formatter1.Format(entry).Equals(result));   
        }

        [TestMethod]
        public void CanCreateFilterFromFactory()
        {
            LogFilterCustomFactory factory = LogFilterCustomFactory.Instance;
            ILogFilter filter = factory.Create(null, new PriorityFilterData("NotFoundFilter", 100), new SystemConfigurationSource(), null);
            Assert.IsNotNull(filter);
            Assert.IsTrue(filter.Filter(new LogEntry()));
        }

        [TestMethod]
        [ExpectedException(typeof(NullReferenceException))]
        public void CanCreateNullTraceSourceFromFactory()
        {
            LogSourceCustomFactory factory = LogSourceCustomFactory.Instance;
            TraceListenerCustomFactory.TraceListenerCache cache = TraceListenerCustomFactory.CreateTraceListenerCache(100);
            LogSource mySource = factory.Create(null, null, new SystemConfigurationSource(), null, cache);
        }

        #endregion

        #region Special Sources Test
        [TestMethod]
        public void LogToNotProcessedTraceSource()
        {
            using (EventLog log = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = log.Entries.Count;
                LogEntry entry = new LogEntry();
                entry.Message = "LogToNotProcessedTraceSource";
                entry.Categories.Add("NoCategorySpecified");
                Logger.Write(entry);
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains(entry.Message));
            }
        }
        #endregion

        #region Expected Exception Tests
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void CanCreateNullTraceListenerFromFactory()
        {
            TraceListenerCustomFactory factory = TraceListenerCustomFactory.Instance;
            factory.CreateObject(null, null, new SystemConfigurationSource(), null);
        }


        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void CanCreateNullFormatterFromFactory()
        {
            LogFormatterCustomFactory factory = LogFormatterCustomFactory.Instance;
            ILogFormatter formatter = (ILogFormatter)factory.CreateObject(null, null, new SystemConfigurationSource(), null);
        }

        [TestMethod]
        [ExpectedException(typeof(NullReferenceException))]
        public void CanCreateNullFilterFromFactory()
        {
            LogFilterCustomFactory factory = LogFilterCustomFactory.Instance;
            ILogFilter filter = factory.Create(null, null, new SystemConfigurationSource(), null);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void LogNullValueLogEntryMessage()
        {
            Logger.Write(new LogEntry(null, "", -1, -1, System.Diagnostics.TraceEventType.Critical, "", null));
        }
        #endregion

        #region Extra Information Tests
        [TestMethod] 
        public void ComplusInformationTest()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("General");
            ComPlusInformationProvider provider = new ComPlusInformationProvider();
            Dictionary<string, object> dict = new Dictionary<string, object>();  
            provider.PopulateDictionary(dict);
            entry.ExtendedProperties = dict;  
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                Logger.Write(entry);
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                string loggedMessage = eventLog.Entries[finalCount - 1].Message;
                Assert.IsTrue(loggedMessage.Contains("ActivityId "));
                Assert.IsTrue(loggedMessage.Contains("DirectCallerAccountName "));   
            }

        }

        [TestMethod]
        public void DebugInformationTest()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("General");
            DebugInformationProvider provider = new DebugInformationProvider();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            provider.PopulateDictionary(dict);
            entry.ExtendedProperties = dict;
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                Logger.Write(entry);
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                string loggedMessage = eventLog.Entries[finalCount - 1].Message;
                Assert.IsTrue(loggedMessage.Contains("StackTrace "));
                Assert.IsTrue(loggedMessage.Contains("Microsoft.Practices.EnterpriseLibrary.Logging.ExtraInformation.DebugInformationProvider.PopulateDictionary"));   
            }
        }

        [TestMethod]
        public void ManagedSecurityInformationTest()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("General");
            ManagedSecurityContextInformationProvider provider = new ManagedSecurityContextInformationProvider();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            provider.PopulateDictionary(dict);
            entry.ExtendedProperties = dict;
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                Logger.Write(entry);
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                string loggedMessage = eventLog.Entries[finalCount - 1].Message;
                Assert.IsTrue(loggedMessage.Contains("AuthenticationType "));
                Assert.IsTrue(loggedMessage.Contains("IsAuthenticated "));   

            }
        }

        [TestMethod]
        public void UnManagedSecurityInformationTest()
        {
            LogEntry entry = new LogEntry();
            entry.Categories.Add("General");
            UnmanagedSecurityContextInformationProvider provider = new UnmanagedSecurityContextInformationProvider();
            Dictionary<string, object> dict = new Dictionary<string, object>();
            provider.PopulateDictionary(dict);
            entry.ExtendedProperties = dict;
            using (EventLog eventLog = new EventLog(HelperClass.GetEventLogName()))
            {
                int initialCount = eventLog.Entries.Count;
                Logger.Write(entry);
                int finalCount = eventLog.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                string loggedMessage = eventLog.Entries[finalCount - 1].Message;
                Assert.IsTrue(loggedMessage.Contains("CurrentUser "));
                Assert.IsTrue(loggedMessage.Contains("ProcessAccountName "));   
            }
        }

#endregion

        #region Additional Tests

        [TestMethod]
        public void LogWriterHasDifferentInstanceusingLogWriterFactory()
        {
            LogWriterFactory factory = new LogWriterFactory();
            LogWriter instance1 = factory.Create();
            LogWriter instance2 = factory.Create();
            Assert.IsFalse(Object.ReferenceEquals(instance1, instance2));
            instance1.Dispose();
            instance2.Dispose();  
        }

        [TestMethod]
        public void LogEntryCloneMethod()
        {
            LogEntry entry = new LogEntry();
            entry.ActivityId = new Guid("AAAABBBBCCCCDDDDAAAABBBBCCCCDDDD");
            entry.AddErrorMessage("LogEntryCloneMethodTest exception");
            entry.Categories.Add("Error");
            entry.EventId = 1;
            entry.ExtendedProperties.Add("key1", "value1");
            entry.Message = "To test the cloning method";
            entry.Priority = 10;
            entry.Severity = TraceEventType.Critical;
            entry.Title = "LogEntryCloneMethodTest";

            LogEntry clonedEntry = (LogEntry)entry.Clone();

            Assert.AreEqual(entry.ActivityIdString, clonedEntry.ActivityIdString);
            Assert.AreEqual(entry.Categories.Count, clonedEntry.Categories.Count);
            Assert.AreEqual(entry.EventId, clonedEntry.EventId);
            Assert.AreEqual(entry.ExtendedProperties.Count, clonedEntry.ExtendedProperties.Count);
            Assert.AreEqual(entry.Message, clonedEntry.Message);
            Assert.AreEqual(entry.Priority, clonedEntry.Priority);
            Assert.AreEqual(entry.ProcessId, clonedEntry.ProcessId);
            Assert.AreEqual(entry.ProcessName, clonedEntry.ProcessName);
            Assert.AreEqual(entry.Severity, clonedEntry.Severity);
            Assert.AreEqual(entry.TimeStamp, clonedEntry.TimeStamp);
            Assert.AreEqual(entry.TimeStampString, clonedEntry.TimeStampString);
            Assert.AreEqual(entry.Title, clonedEntry.Title);
            Assert.AreEqual(entry.Win32ThreadId, clonedEntry.Win32ThreadId);
            Assert.AreEqual(entry.ManagedThreadName, clonedEntry.ManagedThreadName);
            Assert.AreEqual(entry.MachineName, clonedEntry.MachineName);
            Assert.AreEqual(entry.ErrorMessages, clonedEntry.ErrorMessages);
            Assert.AreEqual(entry.AppDomainName, clonedEntry.AppDomainName);

            clonedEntry.Categories.Add("Debug");
            clonedEntry.ExtendedProperties.Add("key2", "value2");
            clonedEntry.ActivityId = new Guid("EEEEFFFFEEEEFFFFEEEEFFFFEEEEFFFF");

            Assert.IsTrue(entry.Categories.Count == 1);
            Assert.IsTrue(clonedEntry.Categories.Count == 2);
            Assert.IsTrue(entry.ExtendedProperties.Count == 1);
            Assert.IsTrue(clonedEntry.ExtendedProperties.Count == 2);
            Assert.IsFalse(entry.ActivityIdString == clonedEntry.ActivityIdString);
        }

        [TestMethod]
        public void MsmqDefaultValuesTest()
        {
            MsmqTraceListenerData msmqData = new MsmqTraceListenerData();
            Assert.IsNotNull(msmqData);
            Assert.AreEqual(msmqData.TimeToBeReceived, System.Messaging.Message.InfiniteTimeout);
            Assert.AreEqual(msmqData.TimeToReachQueue, System.Messaging.Message.InfiniteTimeout);
            Assert.IsTrue(msmqData.Recoverable == false);
            Assert.IsTrue(msmqData.UseDeadLetterQueue == false);
            Assert.IsTrue(msmqData.UseEncryption == false);
            Assert.IsTrue(msmqData.UseAuthentication == false);
            Assert.IsTrue(msmqData.TransactionType == MessageQueueTransactionType.None);
        }

        [TestMethod]
        public void GetFilterTest()
        {
            using (LogWriter writer = EnterpriseLibraryFactory.BuildUp<LogWriter>())
            {
                ILogFilter filter = writer.GetFilter("Priority Filter");
                LogEntry entry = new LogEntry();
                Assert.IsNotNull(filter);
                Assert.IsTrue(filter.Filter(entry));
            }
        }

        [TestMethod]
        public void GetFilterOverloadsTest()
        {
            ILogFilter filter = Logger.GetFilter("Priority Filter");
            Assert.IsTrue(filter.Filter(new LogEntry()));
            ILogFilter filter1 = Logger.GetFilter<PriorityFilter>();
            Assert.IsTrue(filter1.Filter(new LogEntry()));
            ILogFilter filter2 = Logger.GetFilter<PriorityFilter>("Priority Filter");
            Assert.IsTrue(filter2.Filter(new LogEntry()));
        }

        [TestMethod]
        public void LogWriterPropertiesTest()
        {
            LogWriter writer = Logger.Writer;
            Assert.IsTrue(writer.IsLoggingEnabled());
            Assert.IsTrue(writer.IsTracingEnabled());
            Assert.IsTrue(writer.ShouldLog(new LogEntry()));
        }
        #endregion
    }
}
