using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Logging;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation; 
using System.Diagnostics;
using CustomClasses;
namespace LoggerFacade
{
    /// <summary>
    /// Summary description for InstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class InstrumentationTestFixture
    {
        public InstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

       #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

       #region Performance Counter tests
        [TestMethod]
        public void LogtoOneListenerandVerifyTraceListenerEntries()
        {
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName,"Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            Logger.Write(entry);
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogtoOneListenerandVerifyLoggingEventsRaised()
        {
            string counterName = "Logging Events Raised/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            Logger.Write(entry);
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogToAllListenersAndVerifyTraceListenerEntries()
        {
            string counterName = "Trace Listener Entries Written/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            entry.Categories.Add("General");
            entry.Categories.Add("WMICategory");
            entry.Categories.Add("MsmqCategory");
            entry.Categories.Add("FileCategory");
            entry.Categories.Add("SystemLogCategory");
            entry.Categories.Add("RegistryCategory");
            Logger.Write(entry);
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 10);
        }

        [TestMethod]
        public void LogToAllListenersAndVerifyLoggingEventsRaised()
        {
            string counterName = "Logging Events Raised/sec";
            int initialCount = HelperClass.GetCounterValue(counterName, "Total");
            LogEntry entry = new LogEntry();
            entry.Severity = TraceEventType.Error;
            entry.Message = "";
            entry.Categories.Add("DatabaseCategory");
            entry.Categories.Add("General");
            entry.Categories.Add("WMICategory");
            entry.Categories.Add("MsmqCategory");
            entry.Categories.Add("FileCategory");
            entry.Categories.Add("SystemLogCategory");
            entry.Categories.Add("RegistryCategory");
            Logger.Write(entry);
            int loggedCount = HelperClass.GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);
        }

        [TestMethod]
        public void LogToSingleTracerAndVerifyTracerOperationsStarted()
        {
            string counterName = "Trace Operations Started/sec";
            int initialCount = HelperClass.GetCounterValue(counterName,"DatabaseCategory");
            using(Tracer trace=new Tracer("DatabaseCategory")) 
            {

            }
            int finalCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue(finalCount - initialCount == 1);  
        }

        [TestMethod]
        public void LogToMultipleTracerAndVerifyTraceOperationsStarted()
        {
            string counterName = "Trace Operations Started/sec";
            int initialDbCount = HelperClass.GetCounterValue(counterName,"DatabaseCategory");
            int initialEventLogCount=HelperClass.GetCounterValue(counterName,"General");
            int initialMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
            int finalDbCount = 0, finalEventLogCount = 0, finalMsmqCount = 0;
            using (Tracer trace = new Tracer("DatabaseCategory"))
            {
                finalDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");  
                using (Tracer trace1 = new Tracer("General"))
                {
                    finalEventLogCount = HelperClass.GetCounterValue(counterName, "General");  
                    InnerMethod();
                    finalMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");  
                }
            }
            Assert.IsTrue(finalDbCount - initialDbCount == 1);
            Assert.IsTrue(finalEventLogCount - initialEventLogCount == 1);
            Assert.IsTrue(finalMsmqCount - initialMsmqCount == 1);
        }

        [TestMethod]
        public void LogToSingleTracerAndVerifyAvgExecutionTime()
        {
            string counterName = "Avg. Trace Execution Time";
            int initialCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            using (Tracer trace = new Tracer("DatabaseCategory"))
            {

            }
            int finalCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue(finalCount - initialCount > 0);  
        }

        [TestMethod]
        public void LogToMultipleTracerAndVerifyAvgExecutionTime()
        {
            string counterName = "Avg. Trace Execution Time";
            int initialDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            int initialEventLogCount = HelperClass.GetCounterValue(counterName, "General");
            int initialMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
            int finalDbCount = 0, finalEventLogCount = 0, finalMsmqCount = 0;
            using (Tracer trace = new Tracer("DatabaseCategory"))
            {
                Logger.Write(new LogEntry());     
                using (Tracer trace1 = new Tracer("General"))
                {
                    Logger.Write(new LogEntry());
                    InnerMethod();
                    finalMsmqCount = HelperClass.GetCounterValue(counterName, "MsmqCategory");
                }
                finalEventLogCount = HelperClass.GetCounterValue(counterName, "General");
            }
            finalDbCount = HelperClass.GetCounterValue(counterName, "DatabaseCategory");
            Assert.IsTrue((finalDbCount - initialDbCount) > 0);
            Assert.IsTrue((finalEventLogCount - initialEventLogCount) > 0);
            Assert.IsTrue((finalMsmqCount - initialMsmqCount) > 0);
            Assert.IsTrue(finalEventLogCount > finalMsmqCount);
            Assert.IsTrue(finalDbCount > finalEventLogCount);  

        }


        private void InnerMethod()
        {
            using (Tracer trace = new Tracer("MsmqCategory", new Guid()))
            {
                Logger.Write(new LogEntry());
            }
        }
        #endregion
       
    }
}
