using System;
using System.Text;
using System.Diagnostics; 
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Logging;
using CustomClasses;   
namespace LoggerFacade
{
    /// <summary>
    /// Summary description for LoggerOverloadFixture
    /// </summary>
    [TestClass]
    public class LoggerOverloadsFixture
    {
        public LoggerOverloadsFixture()
        {
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void LoggerMessageAndCategoryOverload()
        {
            string message = "LoggerMessageAndCategoryOverload";
            string Category = "General";

            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, Category);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Message.Contains("LoggerMessageAndCategoryOverload"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));  
            }
        }

        [TestMethod]
        public void LoggerMessageCategoryAndExtendedPropOverload()
        {
            string message = "LoggerMessageCategoryAndExtendedPropOverload";
            string category = "General";
            Dictionary<string, object> extendedProp = new Dictionary<string, object>(); 
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category, extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category) );
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }

        [TestMethod]
        public void LoggerTitleMessageCategoryEIdPIdAndExtendedPropOverload()
        {
            string message = "LoggerTitleMessageCategoryAndExtendedPropOverload";
            string category = "General";
            int priority = 4;
            int eventId= 1;
            TraceEventType severity = TraceEventType.Error;  
            string title = "Overload test with parmaters title, category, message, priority, eventid and extended properties";
            Dictionary<string, object> extendedProp = new Dictionary<string, object>();
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category,priority ,eventId,severity ,title, extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category) );
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(title));
                Assert.IsTrue(loggedText.Contains(eventId.ToString())); 
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }

        [TestMethod]
        public void LoggerMessageCategoryPriorityOverload()
        {
            string message = "LoggerMessageCategoryPriorityOverload";
            string category = "General";
            int priority = 3;

            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category, priority);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));     
            }
        }

        [TestMethod]
        public void LoggerMessageCategoryPriorityEventIdOverload()
        {
            string message = "LoggerMessageCategoryPriorityEventIdOverload";
            string category = "General";
            int priority = 2;
            int eventId = 1;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category, priority,eventId);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventId.ToString()));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoryPriorityEventIdSeverityOverload()
        {
            string message = "LoggerMessageCategoryPriorityEventIdSeverityOverload";
            string category = "General";
            int priority = 1;
            int eventId = 1;
            TraceEventType severity = TraceEventType.Warning;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category, priority, eventId,severity);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category));
                Assert.IsTrue(loggedText.Contains(priority.ToString() ));
                Assert.IsTrue(loggedText.Contains(eventId.ToString()));
                Assert.IsTrue(loggedText.Contains(severity.ToString()));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoryPriorityEventIdSeverityTitleOverload()
        {
            string message = "LoggerMessageCategoryPriorityEventIdSeverityTitleOverload";
            string category = "General";
            int priority = 2;
            int eventId = 1;
            TraceEventType severity = TraceEventType.Information;
            string title = "Logger overload test with paramters message, category, priority, eventId, severity and title";
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, category, priority, eventId, severity,title);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message) && loggedText.Contains(category));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventId.ToString()));
                Assert.IsTrue(loggedText.Contains(severity.ToString()));
                Assert.IsTrue(loggedText.Contains(title));
            }
        }

        [TestMethod]
        public void LoggerMessageOverload()
        {
            string message = "LoggerMessageOverload";
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesOverload()
        {
            string message = "LoggerMessageCategoriesOverload";
            IList<string> categories =new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");  
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message,categories);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
            }
        }

        [TestMethod]
        public void LoggerMessageExtendedPropOverload()
        {
            string message = "LoggerMessageExtendedPropOverload";
            Dictionary<string, object> extendedProp = new Dictionary<string, object>();
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message,extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(1, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesExtendedPropOverload()
        {
            string message = "LoggerMessageCategoriesExtendedPropOverload";
            Dictionary<string, object> extendedProp = new Dictionary<string, object>();
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");  
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories,extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesPriorityOverload()
        {
            string message = "LoggerMessageExtendedPropOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            int priority = 1;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
            }
        }


        [TestMethod]
        public void LoggerMessageCategoriesPriorityExtendedPropOverload()
        {
            string message = "LoggerMessageCategoriesPriorityExtendedPropOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            Dictionary<string, object> extendedProp = new Dictionary<string, object>();
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            int priority = 1;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority,extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesPriorityEventIdOverload()
        {
            string message = "LoggerMessageCategoriesPriorityEventIdOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            int priority = 1;
            int eventid = 1;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority, eventid);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventid.ToString()));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesPriorityEventIdSeverityOverload()
        {
            string message = "LoggerMessageCategoriesPriorityEventIdSeverityOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            int priority = 1;
            int eventid = 1;
            TraceEventType severity = TraceEventType.Warning;  
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority, eventid,severity);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventid.ToString()));
                Assert.IsTrue(loggedText.Contains(severity.ToString()));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesPriorityEventIdSeverityTitleOverload()
        {
            string message = "LoggerMessageCategoriesPriorityEventIdSeverityTitleOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            int priority = 1;
            int eventid = 1;
            string title = "Logger overload test with parameters message, categories, priority, eventId, severity and title";
            TraceEventType severity = TraceEventType.Warning;
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority, eventid, severity,title);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventid.ToString()));
                Assert.IsTrue(loggedText.Contains(severity.ToString()));
                Assert.IsTrue(loggedText.Contains(title));
            }
        }

        [TestMethod]
        public void LoggerMessageCategoriesPIdEIdSeverityTitleExtendedPropOverload()
        {
            string message = "LoggerMessageCategoriesPIdEIdSeverityTitleExtendedPropOverload";
            IList<string> categories = new List<string>();
            categories.Add("General");
            categories.Add("EventLogCategory");
            int priority = 1;
            int eventid = 1;
            TraceEventType severity = TraceEventType.Warning;
            string title = "Logger overload test with parameters message, categories, priority, eventId, severity, title and Extended properties";
            Dictionary<string, object> extendedProp = new Dictionary<string, object>();
            extendedProp.Add("AdditionalContext1", "username/password info");
            extendedProp.Add("AdditionalContext2", "process info");
            using (EventLog eventlog = new EventLog(HelperClass.GetEventLogName()))
            {
                int count1 = eventlog.Entries.Count;
                Logger.Write(message, categories, priority, eventid, severity,title,extendedProp);
                int count2 = eventlog.Entries.Count;
                Assert.AreEqual(2, (count2 - count1));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 1].Source.Equals("Enterprise Library Logging Test"));
                Assert.IsTrue(eventlog.Entries[(eventlog.Entries.Count) - 2].Source.Equals("Enterprise Library Logging Test"));
                string loggedText = eventlog.Entries[(eventlog.Entries.Count) - 1].Message;
                Assert.IsTrue(loggedText.Contains(message));
                Assert.IsTrue(loggedText.Contains(categories[0]));
                Assert.IsTrue(loggedText.Contains(categories[1]));
                Assert.IsTrue(loggedText.Contains(priority.ToString()));
                Assert.IsTrue(loggedText.Contains(eventid.ToString()));
                Assert.IsTrue(loggedText.Contains(severity.ToString()));
                Assert.IsTrue(loggedText.Contains(title));
                foreach (string value in extendedProp.Values)
                {
                    Assert.IsTrue(loggedText.Contains(value));
                }
            }
        }
    }
}
