using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using System.Management;
using System.Threading;
using System.Diagnostics;

namespace SecurityCoreFuncTests
{
    /// <summary>
    /// Summary description for AuthRuleInstrumentationTestFixture
    /// </summary>
    [TestClass]
    public class AuthRuleInstrumentationTestFixture
    {
        String wmiText = "";
        bool wmiLogged = false;
        public AuthRuleInstrumentationTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void AuthRuleCheckPerformedCounterTest()
        {
            IIdentity identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal principal = new GenericPrincipal(identity, roles);
            string counterName = "Authorization Requests/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0;

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRule");
            Assert.IsTrue(authorized);

            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount); 
        }

        [TestMethod]
        public void AuthRuleCheckPerformedWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM AuthorizationCheckPerformedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            IIdentity identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal principal = new GenericPrincipal(identity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRule");
            Assert.IsTrue(authorized);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("AuthorizationCheckPerformedEvent") == -1)
            {
                Assert.Fail();
            }
        }

        [TestMethod]
        public void AuthRuleCheckFailedCounterTest()
        {
            IIdentity invalidIdentity = new GenericIdentity("UnauthorizedUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal invalidPrincipal = new GenericPrincipal(invalidIdentity, roles);

            string counterName = "Authorization Requests Denied/sec";
            int initialCount = GetCounterValue(counterName);
            int loggedCount = 0; 
            
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(invalidPrincipal, "TestIdentityRule");
            Assert.IsFalse(authorized);

            loggedCount = GetCounterValue(counterName);
            Assert.AreEqual(loggedCount - 1, initialCount);  
        }

        [TestMethod]
        public void AuthRuleCheckFailedWMITest()
        {
            wmiLogged = false;
            wmiText = "";
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM AuthorizationCheckFailedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start(); 
            
            IIdentity invalidIdentity = new GenericIdentity("UnauthorizedUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal invalidPrincipal = new GenericPrincipal(invalidIdentity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(invalidPrincipal, "TestIdentityRule");
            Assert.IsFalse(authorized);

            int timeoutCount = 10;
            int count = 0;
            bool loopTimed = false;
            while (!wmiLogged && (loopTimed == false))
            {
                Thread.Sleep(100);
                count++;
                if (count == timeoutCount)
                {
                    loopTimed = true;
                }
            }
            watcher.Stop();
            Assert.AreEqual(true, wmiLogged);
            if (wmiText.IndexOf("AuthorizationCheckFailedEvent") == -1)
            {
                Assert.Fail();
            }
        }
        private int GetCounterValue(string counterName)
        {
            //Verify the perf counter
            string categoryName = "Enterprise Library Security Counters";
            string instanceName = new AppDomainNameFormatter().CreateName("Total");
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            wmiLogged = true;
            /// Validate the message here
            wmiText = args.NewEvent.GetText(TextFormat.Mof);
        }

    }
}
