using System;
using System.Text;
using System.Collections.Generic;
using System.Configuration;
using System.Security.Principal;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;




namespace SecurityCoreFuncTests.Authorization
{
    /// <summary>
    /// Summary description for AuthorizationRuleObjectCreationTestFixture
    /// </summary>
    [TestClass]
    public class AuthorizationRuleObjectCreationTestFixture
    {
        public AuthorizationRuleObjectCreationTestFixture()
        {
        
     
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void CreateNamedAuthRuleProviderInConfigFile()
        {
            IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider");
            Assert.IsNotNull(authRuleProvider);

        }
        [TestMethod]
        public void CreateDefaultAuthRuleProviderFromConfigFile()
        {
           IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider();
            Assert.IsNotNull(authRuleProvider);

        }

        [TestMethod]
        [ExpectedException(typeof(ConfigurationErrorsException))]
        public void CreateInvalidAuthRuleProviderFromConfigFile()
        {
            IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("InvalidRule");
           
         
        }


        [TestMethod]
        public void CreateAuthRuleProviderDirect()
        {
           IAuthorizationRule ruleData= new AuthorizationRuleData("TestRule","I:TestUser");
           IDictionary<string,IAuthorizationRule> ruleDict= new Dictionary <string,IAuthorizationRule>();
            ruleDict["TestRule"]= ruleData;
            IAuthorizationRule ruleData1 = new AuthorizationRuleData("TestRule1", "I:TestUser1");
            ruleDict["TestRule1"] = ruleData1;
            IAuthorizationProvider provider = new AuthorizationRuleProvider(ruleDict);
            bool isAuthorized = provider.Authorize(new GenericPrincipal(new GenericIdentity("TestUser"), new string[]{}),"TestRule");
            Assert.IsTrue(isAuthorized);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void CreateAuthRuleProviderDirectWithNullParam()
        {
            IDictionary<string, IAuthorizationRule> ruleDict = null;
            IAuthorizationProvider provider = new AuthorizationRuleProvider(ruleDict);
            bool isAuthorized = provider.Authorize(new GenericPrincipal(new GenericIdentity("TestUser"), new string[] { }), "TestRule");
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AuthorizeProviderWithNullIdentity()
        {
            IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider();
            authRuleProvider.Authorize(null, "NoRule");  
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void AuthorizeProviderWithNullRule()
        {
            IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider();
            authRuleProvider.Authorize(new WindowsPrincipal(WindowsIdentity.GetAnonymous()) , null);
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void AuthorizeProviderWithInvalidRule()
        {
            IAuthorizationProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider();
            authRuleProvider.Authorize(new WindowsPrincipal(WindowsIdentity.GetAnonymous()), "NonExistingRule");
        }

    }
}
