using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using System.Security.Principal;
using Microsoft.Practices.EnterpriseLibrary.Security;
using Microsoft.Practices.EnterpriseLibrary.Security.Configuration;

namespace SecurityCoreFuncTests
{
    /// <summary>
    /// Summary description for AuthorizationRuleTestFixture
    /// </summary>
    [TestClass]
    public class AuthorizationRuleTestFixture
    {
        IIdentity identity;
        private static IPrincipal principal;
        public AuthorizationRuleTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestInitialize()]
        public void SetUp()
        {
            identity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "Manager" };
            principal = new GenericPrincipal(identity, roles);
        }

        [TestMethod]
        public void AuthorizeIdentityTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeAnonymousIdentityTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(new WindowsPrincipal(WindowsIdentity.GetAnonymous()) , "TestAnonymousRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeAnyRoleTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestAnyRoleRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeNotOperatorRuleTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestNotOperatorRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeInvalidIdentityTest()
        {
            IIdentity invalidIdentity = new GenericIdentity("UnauthorizedUser");
            string[] roles = new string[] { "Manager" };
            IPrincipal invalidPrincipal = new GenericPrincipal(invalidIdentity, roles); 
            
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(invalidPrincipal, "TestIdentityRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityNewRoleTest()
        {
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeInvalidRoleTest()
        {
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityOrRoleTest()
        {
            // Testing "Identity or Role" rule for authorizing identity.
            IIdentity newIdentity = new GenericIdentity("TestUser");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRule");
            Assert.IsTrue(authorized, "Authorizing Identity using Identity Or Role rule failed");

            // Testing "Identity or Role" rule for authorizing role.
            newIdentity = new GenericIdentity("NewUser");
            roles = new string[] { "Manager" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRule");
            Assert.IsTrue(authorized, "Authorizing Role using Identity Or Role rule failed");
        }

        [TestMethod]
        public void AuthorizeInvalidIdentityOrRoleTest()
        {
            // Testing "Identity or Role" rule for authorizing identity.
            IIdentity newIdentity = new GenericIdentity("NewUser");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORRoleRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityAndRoleTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityAndRoleRule");
            Assert.IsTrue(authorized);

            // Testing "Identity And Role" rule with invalid role.
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityAndRoleRule");
            Assert.IsFalse(authorized);

        }

        [TestMethod]
        public void AuthorizeMultipleIdentitiesTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityORIdentityRule");
            Assert.IsTrue(authorized);

            IIdentity newIdentity = new GenericIdentity("Me");
            string[] roles = new string[] { "NewRole" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORIdentityRule");
            Assert.IsTrue(authorized);
            
            //Testing invalid identitity
            newIdentity = new GenericIdentity("InvalidUser");
            roles = new string[] { "NewRole" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestIdentityORIdentityRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeIdentityAndIdentityTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestIdentityAndIdentityRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeMultipleRolesTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleORRoleRule");
            Assert.IsTrue(authorized);

            IIdentity newIdentity = new GenericIdentity("Me");
            string[] roles = new string[] { "Employee" };
            IPrincipal newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleORRoleRule");
            Assert.IsTrue(authorized);

            //Testing invalid identitity
            newIdentity = new GenericIdentity("InvalidUser");
            roles = new string[] { "NewRole" };
            newPrincipal = new GenericPrincipal(newIdentity, roles);

            authorized = authRuleProvider.Authorize(newPrincipal, "TestRoleORRoleRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleAndRoleTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleAndRoleRule");
            Assert.IsFalse(authorized);
        }

        [TestMethod]
        public void AuthorizeRoleOrIdentityTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestRoleOrIdentityRule");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void AuthorizeRuleNameWithSpacesTest()
        {
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(principal, "TestRuleName With Spaces");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        [ExpectedException(typeof(SyntaxException))]
        public void AuthorizeRuleWithSpacesTest()
        {
            string[] roles = new string[] { "Test Manager" };
            IPrincipal newPrincipal = new GenericPrincipal(identity, roles); 
            
            AuthorizationRuleProvider authRuleProvider = AuthorizationFactory.GetAuthorizationProvider("DefaultRuleProvider") as AuthorizationRuleProvider;
            bool authorized = authRuleProvider.Authorize(newPrincipal, "TestRule With Spaces");
            Assert.IsTrue(authorized);
        }

        [TestMethod]
        public void IdentityExpressionDirectTest()
        {
            IdentityExpression expression = new IdentityExpression("?");
            Assert.IsTrue(expression.Evaluate(new WindowsPrincipal(WindowsIdentity.GetAnonymous())));  
        }

        [TestMethod]
        public void RoleExpressionDirectTest()
        {
            RoleExpression expression = new RoleExpression("*"); 
            Assert.IsTrue(expression.Evaluate(new WindowsPrincipal(WindowsIdentity.GetAnonymous())));
        }

    }
}
